from insights.tests import context_wrap
from insights.parsers import tomcat_xml
from insights.parsers.tomcat_xml import TomcatWebXml
from insights.parsers.tomcat_xml import TomcatServerXml
import doctest

web_xml_content = """
<?xml version="1.0" encoding="ISO-8859-1"?>
<!--
  Licensed to the Apache Software Foundation (ASF) under one or more
  contributor license agreements.  See the NOTICE file distributed with
  this work for additional information regarding copyright ownership.
  The ASF licenses this file to You under the Apache License, Version 2.0
  (the "License"); you may not use this file except in compliance with
  the License.  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
-->
<web-app xmlns="http://java.sun.com/xml/ns/javaee"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://java.sun.com/xml/ns/javaee http://java.sun.com/xml/ns/javaee/web-app_2_5.xsd"
    version="2.5">

  <!-- ======================== Introduction ============================== -->
  <!-- This document defines default values for *all* web applications      -->
  <!-- loaded into this instance of Tomcat.  As each application is         -->
  <!-- deployed, this file is processed, followed by the                    -->
  <!-- "/WEB-INF/web.xml" deployment descriptor from your own               -->
  <!-- applications.                                                        -->
  <!--                                                                      -->
  <!-- WARNING:  Do not configure application-specific resources here!      -->
  <!-- They should go in the "/WEB-INF/web.xml" file in your application.   -->


  <!-- ================== Built In Servlet Definitions ==================== -->


  <!-- The default servlet for all web applications, that serves static     -->
  <!-- resources.  It processes all requests that are not mapped to other   -->
  <!-- servlets with servlet mappings (defined either here or in your own   -->
  <!-- web.xml file.  This servlet supports the following initialization    -->
  <!-- parameters (default values are in square brackets):                  -->
  <!--                                                                      -->
  <!--   debug               Debugging detail level for messages logged     -->
  <!--                       by this servlet.  [0]                          -->
  <!--                                                                      -->
  <!--   fileEncoding        Encoding to be used to read static resources   -->
  <!--                       [platform default]                             -->
  <!--                                                                      -->
  <!--   input               Input buffer size (in bytes) when reading      -->
  <!--                       resources to be served.  [2048]                -->
  <!--                                                                      -->
  <!--   listings            Should directory listings be produced if there -->
  <!--                       is no welcome file in this directory?  [false] -->
  <!--                       WARNING: Listings for directories with many    -->
  <!--                       entries can be slow and may consume            -->
  <!--                       significant proportions of server resources.   -->
  <!--                                                                      -->
  <!--   output              Output buffer size (in bytes) when writing     -->
  <!--                       resources to be served.  [2048]                -->
  <!--                                                                      -->
  <!--   readonly            Is this context "read only", so HTTP           -->
  <!--                       commands like PUT and DELETE are               -->
  <!--                       rejected?  [true]                              -->
  <!--                                                                      -->
  <!--   readmeFile          File name to display with the directory        -->
  <!--                       contents. [null]                               -->
  <!--                                                                      -->
  <!--   sendfileSize        If the connector used supports sendfile, this  -->
  <!--                       represents the minimal file size in KB for     -->
  <!--                       which sendfile will be used. Use a negative    -->
  <!--                       value to always disable sendfile.  [48]        -->
  <!--                                                                      -->
  <!--   useAcceptRanges     Should the Accept-Ranges header be included    -->
  <!--                       in responses where appropriate? [true]         -->
  <!--                                                                      -->
  <!--  For directory listing customization. Checks localXsltFile, then     -->
  <!--  globalXsltFile, then defaults to original behavior.                 -->
  <!--                                                                      -->
  <!--   localXsltFile       Make directory listings an XML doc and         -->
  <!--                       pass the result to this style sheet residing   -->
  <!--                       in that directory. This overrides              -->
  <!--                        globalXsltFile[null]                          -->
  <!--                                                                      -->
  <!--   globalXsltFile      Site wide configuration version of             -->
  <!--                       localXsltFile. This argument must either       -->
  <!--                        an aboslute or relative ( to either           -->
  <!--                        CATALINA_BASE/conf or CATALINA_HOME/conf     -->
  <!--                        path that points to the location below       -->
  <!--                        CATALINA_BASE/conf (checked first) or        -->
  <!--                        $CATLINA_HOME/conf (checked second).         -->
  <!--                                                                      -->
  <!--                                                                      -->

    <servlet>
        <servlet-name>default</servlet-name>
        <servlet-class>org.apache.catalina.servlets.DefaultServlet</servlet-class>
        <init-param>
            <param-name>debug</param-name>
            <param-value>0</param-value>
        </init-param>
        <init-param>
            <param-name>listings</param-name>
            <param-value>false</param-value>
        </init-param>
        <load-on-startup>1</load-on-startup>
    </servlet>


  <!-- This servlet has been deprecated due to security concerns. Servlets  -->
  <!-- should be explicitly mapped in web.xml                               -->
  <!--                                                                      -->
  <!-- The "invoker" servlet, which executes anonymous servlet classes      -->
  <!-- that have not been defined in a web.xml file.  Traditionally, this   -->
  <!-- servlet is mapped to the URL pattern "/servlet/*", but you can map   -->
  <!-- it to other patterns as well.  The extra path info portion of such a -->
  <!-- request must be the fully qualified class name of a Java class that  -->
  <!-- implements Servlet (or extends HttpServlet), or the servlet name     -->
  <!-- of an existing servlet definition.     This servlet supports the     -->
  <!-- following initialization parameters (default values are in square    -->
  <!-- brackets):                                                           -->
  <!--                                                                      -->
  <!--   debug               Debugging detail level for messages logged     -->
  <!--                       by this servlet.  [0]                          -->

<!--
    <servlet>
        <servlet-name>invoker</servlet-name>
        <servlet-class>
          org.apache.catalina.servlets.InvokerServlet
        </servlet-class>
        <init-param>
            <param-name>debug</param-name>
            <param-value>0</param-value>
        </init-param>
        <load-on-startup>2</load-on-startup>
    </servlet>
-->


  <!-- The JSP page compiler and execution servlet, which is the mechanism  -->
  <!-- used by Tomcat to support JSP pages.  Traditionally, this servlet    -->
  <!-- is mapped to the URL pattern "*.jsp".  This servlet supports the     -->
  <!-- following initialization parameters (default values are in square    -->
  <!-- brackets):                                                           -->
  <!--                                                                      -->
  <!--   checkInterval       If development is false and checkInterval is   -->
  <!--                       greater than zero, background compilations are -->
  <!--                       enabled. checkInterval is the time in seconds  -->
  <!--                       between checks to see if a JSP page (and its   -->
  <!--                       dependent files) needs to  be recompiled. [0]  -->
  <!--                                                                      -->
  <!--   classdebuginfo      Should the class file be compiled with         -->
  <!--                       debugging information?  [true]                 -->
  <!--                                                                      -->
  <!--   classpath           What class path should I use while compiling   -->
  <!--                       generated servlets?  [Created dynamically      -->
  <!--                       based on the current web application]          -->
  <!--                                                                      -->
  <!--   compiler            Which compiler Ant should use to compile JSP   -->
  <!--                       pages.  See the jasper documentation for more  -->
  <!--                       information.                                   -->
  <!--                                                                      -->
  <!--   compilerSourceVM    Compiler source VM. [1.5]                      -->
  <!--                                                                      -->
  <!--   compilerTargetVM    Compiler target VM. [1.5]                      -->
  <!--                                                                      -->
  <!--   development         Is Jasper used in development mode? If true,   -->
  <!--                       the frequency at which JSPs are checked for    -->
  <!--                       modification may be specified via the          -->
  <!--                       modificationTestInterval parameter. [true]     -->
  <!--                                                                      -->
  <!--   displaySourceFragment                                              -->
  <!--                       Should a source fragment be included in        -->
  <!--                       exception messages? [true]                     -->
  <!--                                                                      -->
  <!--   dumpSmap            Should the SMAP info for JSR45 debugging be    -->
  <!--                       dumped to a file? [false]                      -->
  <!--                       False if suppressSmap is true                  -->
  <!--                                                                      -->
  <!--   enablePooling       Determines whether tag handler pooling is      -->
  <!--                       enabled. This is a compilation option. It will -->
  <!--                       not alter the behaviour of JSPs that have      -->
  <!--                       already been compiled. [true]                  -->
  <!--                                                                      -->
  <!--   engineOptionsClass  Allows specifying the Options class used to    -->
  <!--                       configure Jasper. If not present, the default  -->
  <!--                       EmbeddedServletOptions will be used.           -->
  <!--                                                                      -->
  <!--   errorOnUseBeanInvalidClassAttribute                                -->
  <!--                       Should Jasper issue an error when the value of -->
  <!--                       the class attribute in an useBean action is    -->
  <!--                       not a valid bean class?  [true]                -->
  <!--                                                                      -->
  <!--   fork                Tell Ant to fork compiles of JSP pages so that -->
  <!--                       a separate JVM is used for JSP page compiles   -->
  <!--                       from the one Tomcat is running in. [true]      -->
  <!--                                                                      -->
  <!--   genStrAsCharArray   Should text strings be generated as char       -->
  <!--                       arrays, to improve performance in some cases?  -->
  <!--                       [false]                                        -->
  <!--                                                                      -->
  <!--   ieClassId           The class-id value to be sent to Internet      -->
  <!--                       Explorer when using <jsp:plugin> tags.         -->
  <!--                       [clsid:8AD9C840-044E-11D1-B3E9-00805F499D93]   -->
  <!--                                                                      -->
  <!--   javaEncoding        Java file encoding to use for generating java  -->
  <!--                       source files. [UTF8]                           -->
  <!--                                                                      -->
  <!--   keepgenerated       Should we keep the generated Java source code  -->
  <!--                       for each page instead of deleting it? [true]   -->
  <!--                                                                      -->
  <!--   mappedfile          Should we generate static content with one     -->
  <!--                       print statement per input line, to ease        -->
  <!--                       debugging?  [true]                             -->
  <!--                                                                      -->
  <!--   modificationTestInterval                                           -->
  <!--                       Causes a JSP (and its dependent files) to not  -->
  <!--                       be checked for modification during the         -->
  <!--                       specified time interval (in seconds) from the  -->
  <!--                       last time the JSP was checked for              -->
  <!--                       modification. A value of 0 will cause the JSP  -->
  <!--                       to be checked on every access.                 -->
  <!--                       Used in development mode only. [4]             -->
  <!--                                                                      -->
  <!--   scratchdir          What scratch directory should we use when      -->
  <!--                       compiling JSP pages?  [default work directory  -->
  <!--                       for the current web application]               -->
  <!--                                                                      -->
  <!--   suppressSmap        Should the generation of SMAP info for JSR45   -->
  <!--                       debugging be suppressed?  [false]              -->
  <!--                                                                      -->
  <!--   trimSpaces          Should white spaces in template text between   -->
  <!--                       actions or directives be trimmed?  [false]     -->
  <!--                                                                      -->
  <!--   xpoweredBy          Determines whether X-Powered-By response       -->
  <!--                       header is added by generated servlet  [false]  -->
  <!--                                                                      -->
  <!-- If you wish to use Jikes to compile JSP pages:                       -->
  <!--   Please see the "Using Jikes" section of the Jasper-HowTo           -->
  <!--   page in the Tomcat documentation.                                  -->

    <servlet>
        <servlet-name>jsp</servlet-name>
        <servlet-class>org.apache.jasper.servlet.JspServlet</servlet-class>
        <init-param>
            <param-name>fork</param-name>
            <param-value>false</param-value>
        </init-param>
        <init-param>
            <param-name>xpoweredBy</param-name>
            <param-value>false</param-value>
        </init-param>
        <init-param>
            <param-name>development</param-name>
            <param-value>false</param-value>
        </init-param>
        <load-on-startup>3</load-on-startup>
    </servlet>


  <!-- NOTE: An SSI Filter is also available as an alternative SSI          -->
  <!-- implementation. Use either the Servlet or the Filter but NOT both.   -->
  <!--                                                                      -->
  <!-- Server Side Includes processing servlet, which processes SSI         -->
  <!-- directives in HTML pages consistent with similar support in web      -->
  <!-- servers like Apache.  Traditionally, this servlet is mapped to the   -->
  <!-- URL pattern "*.shtml".  This servlet supports the following          -->
  <!-- initialization parameters (default values are in square brackets):   -->
  <!--                                                                      -->
  <!--   buffered            Should output from this servlet be buffered?   -->
  <!--                       (0=false, 1=true)  [0]                         -->
  <!--                                                                      -->
  <!--   debug               Debugging detail level for messages logged     -->
  <!--                       by this servlet.  [0]                          -->
  <!--                                                                      -->
  <!--   expires             The number of seconds before a page with SSI   -->
  <!--                       directives will expire.  [No default]          -->
  <!--                                                                      -->
  <!--   isVirtualWebappRelative                                            -->
  <!--                       Should "virtual" paths be interpreted as       -->
  <!--                       relative to the context root, instead of       -->
  <!--                       the server root?  (0=false, 1=true) [0]        -->
  <!--                                                                      -->
  <!--   inputEncoding       The encoding to assume for SSI resources if    -->
  <!--                       one is not available from the resource.        -->
  <!--                       [Platform default]                             -->
  <!--                                                                      -->
  <!--   outputEncoding      The encoding to use for the page that results  -->
  <!--                       from the SSI processing. [UTF-8]               -->

<!--
    <servlet>
        <servlet-name>ssi</servlet-name>
        <servlet-class>
          org.apache.catalina.ssi.SSIServlet
        </servlet-class>
        <init-param>
          <param-name>buffered</param-name>
          <param-value>1</param-value>
        </init-param>
        <init-param>
          <param-name>debug</param-name>
          <param-value>0</param-value>
        </init-param>
        <init-param>
          <param-name>expires</param-name>
          <param-value>666</param-value>
        </init-param>
        <init-param>
          <param-name>isVirtualWebappRelative</param-name>
          <param-value>0</param-value>
        </init-param>
        <load-on-startup>4</load-on-startup>
    </servlet>
-->


  <!-- Common Gateway Includes (CGI) processing servlet, which supports     -->
  <!-- execution of external applications that conform to the CGI spec      -->
  <!-- requirements.  Typically, this servlet is mapped to the URL pattern  -->
  <!-- "/cgi-bin/*", which means that any CGI applications that are         -->
  <!-- executed must be present within the web application.  This servlet   -->
  <!-- supports the following initialization parameters (default values     -->
  <!-- are in square brackets):                                             -->
  <!--                                                                      -->
  <!--   cgiPathPrefix        The CGI search path will start at             -->
  <!--                        webAppRootDir + File.separator + this prefix. -->
  <!--                        [WEB-INF/cgi]                                 -->
  <!--                                                                      -->
  <!--   debug                Debugging detail level for messages logged    -->
  <!--                        by this servlet.  [0]                         -->
  <!--                                                                      -->
  <!--   executable           Name of the executable used to run the        -->
  <!--                        script. [perl]                                -->
  <!--                                                                      -->
  <!--   parameterEncoding    Name of parameter encoding to be used with    -->
  <!--                        CGI servlet.                                  -->
  <!--                        [System.getProperty("file.encoding","UTF-8")] -->
  <!--                                                                      -->
  <!--   passShellEnvironment Should the shell environment variables (if    -->
  <!--                        any) be passed to the CGI script? [false]     -->
  <!--                                                                      -->
  <!--   stderrTimeout        The time (in milliseconds) to wait for the    -->
  <!--                        reading of stderr to complete before          -->
  <!--                        terminating the CGI process. [2000]           -->

<!--
    <servlet>
        <servlet-name>cgi</servlet-name>
        <servlet-class>org.apache.catalina.servlets.CGIServlet</servlet-class>
        <init-param>
          <param-name>debug</param-name>
          <param-value>0</param-value>
        </init-param>
        <init-param>
          <param-name>cgiPathPrefix</param-name>
          <param-value>WEB-INF/cgi</param-value>
        </init-param>
         <load-on-startup>5</load-on-startup>
    </servlet>
-->


  <!-- ================ Built In Servlet Mappings ========================= -->


  <!-- The servlet mappings for the built in servlets defined above.  Note  -->
  <!-- that, by default, the CGI and SSI servlets are *not* mapped.  You    -->
  <!-- must uncomment these mappings (or add them to your application's own -->
  <!-- web.xml deployment descriptor) to enable these services              -->

    <!-- The mapping for the default servlet -->
    <servlet-mapping>
        <servlet-name>default</servlet-name>
        <url-pattern>/</url-pattern>
    </servlet-mapping>

    <!-- The mapping for the deprecated invoker servlet -->
<!--
    <servlet-mapping>
        <servlet-name>invoker</servlet-name>
        <url-pattern>/servlet/*</url-pattern>
    </servlet-mapping>
-->

    <!-- The mapping for the JSP servlet -->
    <servlet-mapping>
        <servlet-name>jsp</servlet-name>
        <url-pattern>*.jsp</url-pattern>
    </servlet-mapping>

    <servlet-mapping>
        <servlet-name>jsp</servlet-name>
        <url-pattern>*.jspx</url-pattern>
    </servlet-mapping>

    <!-- The mapping for the SSI servlet -->
<!--
    <servlet-mapping>
        <servlet-name>ssi</servlet-name>
        <url-pattern>*.shtml</url-pattern>
    </servlet-mapping>
-->

    <!-- The mapping for the CGI Gateway servlet -->

<!--
    <servlet-mapping>
        <servlet-name>cgi</servlet-name>
        <url-pattern>/cgi-bin/*</url-pattern>
    </servlet-mapping>
-->


  <!-- ================== Built In Filter Definitions ===================== -->

  <!-- NOTE: An SSI Servlet is also available as an alternative SSI         -->
  <!-- implementation. Use either the Servlet or the Filter but NOT both.   -->
  <!--                                                                      -->
  <!-- Server Side Includes processing filter, which processes SSI          -->
  <!-- directives in HTML pages consistent with similar support in web      -->
  <!-- servers like Apache.  Traditionally, this filter is mapped to the    -->
  <!-- URL pattern "*.shtml", though it can be mapped to "*" as it will     -->
  <!-- selectively enable/disable SSI processing based on mime types. For   -->
  <!-- this to work you will need to uncomment the .shtml mime type         -->
  <!-- definition towards the bottom of this file.                          -->
  <!-- The contentType init param allows you to apply SSI processing to JSP -->
  <!-- pages, javascript, or any other content you wish.  This filter       -->
  <!-- supports the following initialization parameters (default values are -->
  <!-- in square brackets):                                                 -->
  <!--                                                                      -->
  <!--   contentType         A regex pattern that must be matched before    -->
  <!--                       SSI processing is applied.                     -->
  <!--                       [text/x-server-parsed-html(;.*)?]              -->
  <!--                                                                      -->
  <!--   debug               Debugging detail level for messages logged     -->
  <!--                       by this servlet.  [0]                          -->
  <!--                                                                      -->
  <!--   expires             The number of seconds before a page with SSI   -->
  <!--                       directives will expire.  [No default]          -->
  <!--                                                                      -->
  <!--   isVirtualWebappRelative                                            -->
  <!--                       Should "virtual" paths be interpreted as       -->
  <!--                       relative to the context root, instead of       -->
  <!--                       the server root?  (0=false, 1=true) [0]        -->

<!--
    <filter>
        <filter-name>ssi</filter-name>
        <filter-class>
          org.apache.catalina.ssi.SSIFilter
        </filter-class>
        <init-param>
          <param-name>contentType</param-name>
          <param-value>text/x-server-parsed-html(;.*)?</param-value>
        </init-param>
        <init-param>
          <param-name>debug</param-name>
          <param-value>0</param-value>
        </init-param>
        <init-param>
          <param-name>expires</param-name>
          <param-value>666</param-value>
        </init-param>
        <init-param>
          <param-name>isVirtualWebappRelative</param-name>
          <param-value>0</param-value>
        </init-param>
    </filter>
-->


  <!-- ==================== Built In Filter Mappings ====================== -->

  <!-- The mapping for the SSI Filter -->
<!--
    <filter-mapping>
        <filter-name>ssi</filter-name>
        <url-pattern>*.shtml</url-pattern>
    </filter-mapping>
-->


  <!-- ==================== Default Session Configuration ================= -->
  <!-- You can set the default session timeout (in minutes) for all newly   -->
  <!-- created sessions by modifying the value below.                       -->

    <session-config>
        <session-timeout>30</session-timeout>
    </session-config>


  <!-- ===================== Default MIME Type Mappings =================== -->
  <!-- When serving static resources, Tomcat will automatically generate    -->
  <!-- a "Content-Type" header based on the resource's filename extension,  -->
  <!-- based on these mappings.  Additional mappings can be added here (to  -->
  <!-- apply to all web applications), or in your own application's web.xml -->
  <!-- deployment descriptor.                                               -->

    <mime-mapping>
        <extension>abs</extension>
        <mime-type>audio/x-mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ai</extension>
        <mime-type>application/postscript</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>aif</extension>
        <mime-type>audio/x-aiff</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>aifc</extension>
        <mime-type>audio/x-aiff</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>aiff</extension>
        <mime-type>audio/x-aiff</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>aim</extension>
        <mime-type>application/x-aim</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>art</extension>
        <mime-type>image/x-jg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>asf</extension>
        <mime-type>video/x-ms-asf</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>asx</extension>
        <mime-type>video/x-ms-asf</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>au</extension>
        <mime-type>audio/basic</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>avi</extension>
        <mime-type>video/x-msvideo</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>avx</extension>
        <mime-type>video/x-rad-screenplay</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>bcpio</extension>
        <mime-type>application/x-bcpio</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>bin</extension>
        <mime-type>application/octet-stream</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>bmp</extension>
        <mime-type>image/bmp</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>body</extension>
        <mime-type>text/html</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>cdf</extension>
        <mime-type>application/x-cdf</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>cer</extension>
        <mime-type>application/x-x509-ca-cert</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>class</extension>
        <mime-type>application/java</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>cpio</extension>
        <mime-type>application/x-cpio</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>csh</extension>
        <mime-type>application/x-csh</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>css</extension>
        <mime-type>text/css</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>dib</extension>
        <mime-type>image/bmp</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>doc</extension>
        <mime-type>application/msword</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>dtd</extension>
        <mime-type>application/xml-dtd</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>dv</extension>
        <mime-type>video/x-dv</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>dvi</extension>
        <mime-type>application/x-dvi</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>eps</extension>
        <mime-type>application/postscript</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>etx</extension>
        <mime-type>text/x-setext</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>exe</extension>
        <mime-type>application/octet-stream</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>gif</extension>
        <mime-type>image/gif</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>gtar</extension>
        <mime-type>application/x-gtar</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>gz</extension>
        <mime-type>application/x-gzip</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>hdf</extension>
        <mime-type>application/x-hdf</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>hqx</extension>
        <mime-type>application/mac-binhex40</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>htc</extension>
        <mime-type>text/x-component</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>htm</extension>
        <mime-type>text/html</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>html</extension>
        <mime-type>text/html</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>hqx</extension>
        <mime-type>application/mac-binhex40</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ief</extension>
        <mime-type>image/ief</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>jad</extension>
        <mime-type>text/vnd.sun.j2me.app-descriptor</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>jar</extension>
        <mime-type>application/java-archive</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>java</extension>
        <mime-type>text/plain</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>jnlp</extension>
        <mime-type>application/x-java-jnlp-file</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>jpe</extension>
        <mime-type>image/jpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>jpeg</extension>
        <mime-type>image/jpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>jpg</extension>
        <mime-type>image/jpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>js</extension>
        <mime-type>text/javascript</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>jsf</extension>
        <mime-type>text/plain</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>jspf</extension>
        <mime-type>text/plain</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>kar</extension>
        <mime-type>audio/x-midi</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>latex</extension>
        <mime-type>application/x-latex</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>m3u</extension>
        <mime-type>audio/x-mpegurl</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mac</extension>
        <mime-type>image/x-macpaint</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>man</extension>
        <mime-type>application/x-troff-man</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mathml</extension>
        <mime-type>application/mathml+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>me</extension>
        <mime-type>application/x-troff-me</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mid</extension>
        <mime-type>audio/x-midi</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>midi</extension>
        <mime-type>audio/x-midi</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mif</extension>
        <mime-type>application/x-mif</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mov</extension>
        <mime-type>video/quicktime</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>movie</extension>
        <mime-type>video/x-sgi-movie</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mp1</extension>
        <mime-type>audio/x-mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mp2</extension>
        <mime-type>audio/x-mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mp3</extension>
        <mime-type>audio/x-mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mp4</extension>
        <mime-type>video/mp4</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mpa</extension>
        <mime-type>audio/x-mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mpe</extension>
        <mime-type>video/mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mpeg</extension>
        <mime-type>video/mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mpega</extension>
        <mime-type>audio/x-mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mpg</extension>
        <mime-type>video/mpeg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>mpv2</extension>
        <mime-type>video/mpeg2</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ms</extension>
        <mime-type>application/x-wais-source</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>nc</extension>
        <mime-type>application/x-netcdf</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>oda</extension>
        <mime-type>application/oda</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Database -->
        <extension>odb</extension>
        <mime-type>application/vnd.oasis.opendocument.database</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Chart -->
        <extension>odc</extension>
        <mime-type>application/vnd.oasis.opendocument.chart</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Formula -->
        <extension>odf</extension>
        <mime-type>application/vnd.oasis.opendocument.formula</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Drawing -->
        <extension>odg</extension>
        <mime-type>application/vnd.oasis.opendocument.graphics</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Image -->
        <extension>odi</extension>
        <mime-type>application/vnd.oasis.opendocument.image</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Master Document -->
        <extension>odm</extension>
        <mime-type>application/vnd.oasis.opendocument.text-master</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Presentation -->
        <extension>odp</extension>
        <mime-type>application/vnd.oasis.opendocument.presentation</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Spreadsheet -->
        <extension>ods</extension>
        <mime-type>application/vnd.oasis.opendocument.spreadsheet</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Text -->
        <extension>odt</extension>
        <mime-type>application/vnd.oasis.opendocument.text</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ogg</extension>
        <mime-type>application/ogg</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Drawing Template -->
        <extension>otg </extension>
        <mime-type>application/vnd.oasis.opendocument.graphics-template</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- HTML Document Template -->
        <extension>oth</extension>
        <mime-type>application/vnd.oasis.opendocument.text-web</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Presentation Template -->
        <extension>otp</extension>
        <mime-type>application/vnd.oasis.opendocument.presentation-template</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Spreadsheet Template -->
        <extension>ots</extension>
        <mime-type>application/vnd.oasis.opendocument.spreadsheet-template </mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- OpenDocument Text Template -->
        <extension>ott</extension>
        <mime-type>application/vnd.oasis.opendocument.text-template</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pbm</extension>
        <mime-type>image/x-portable-bitmap</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pct</extension>
        <mime-type>image/pict</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pdf</extension>
        <mime-type>application/pdf</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pgm</extension>
        <mime-type>image/x-portable-graymap</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pic</extension>
        <mime-type>image/pict</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pict</extension>
        <mime-type>image/pict</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pls</extension>
        <mime-type>audio/x-scpls</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>png</extension>
        <mime-type>image/png</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pnm</extension>
        <mime-type>image/x-portable-anymap</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pnt</extension>
        <mime-type>image/x-macpaint</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ppm</extension>
        <mime-type>image/x-portable-pixmap</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ppt</extension>
        <mime-type>application/vnd.ms-powerpoint</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>pps</extension>
        <mime-type>application/vnd.ms-powerpoint</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ps</extension>
        <mime-type>application/postscript</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>psd</extension>
        <mime-type>image/x-photoshop</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>qt</extension>
        <mime-type>video/quicktime</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>qti</extension>
        <mime-type>image/x-quicktime</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>qtif</extension>
        <mime-type>image/x-quicktime</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ras</extension>
        <mime-type>image/x-cmu-raster</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>rdf</extension>
        <mime-type>application/rdf+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>rgb</extension>
        <mime-type>image/x-rgb</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>rm</extension>
        <mime-type>application/vnd.rn-realmedia</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>roff</extension>
        <mime-type>application/x-troff</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>rtf</extension>
        <mime-type>application/rtf</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>rtx</extension>
        <mime-type>text/richtext</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>sh</extension>
        <mime-type>application/x-sh</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>shar</extension>
        <mime-type>application/x-shar</mime-type>
    </mime-mapping>
<!--
    <mime-mapping>
        <extension>shtml</extension>
        <mime-type>text/x-server-parsed-html</mime-type>
    </mime-mapping>
-->
    <mime-mapping>
        <extension>smf</extension>
        <mime-type>audio/x-midi</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>sit</extension>
        <mime-type>application/x-stuffit</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>snd</extension>
        <mime-type>audio/basic</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>src</extension>
        <mime-type>application/x-wais-source</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>sv4cpio</extension>
        <mime-type>application/x-sv4cpio</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>sv4crc</extension>
        <mime-type>application/x-sv4crc</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>svg</extension>
        <mime-type>image/svg+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>svgz</extension>
        <mime-type>image/svg+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>swf</extension>
        <mime-type>application/x-shockwave-flash</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>t</extension>
        <mime-type>application/x-troff</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>tar</extension>
        <mime-type>application/x-tar</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>tcl</extension>
        <mime-type>application/x-tcl</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>tex</extension>
        <mime-type>application/x-tex</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>texi</extension>
        <mime-type>application/x-texinfo</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>texinfo</extension>
        <mime-type>application/x-texinfo</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>tif</extension>
        <mime-type>image/tiff</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>tiff</extension>
        <mime-type>image/tiff</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>tr</extension>
        <mime-type>application/x-troff</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>tsv</extension>
        <mime-type>text/tab-separated-values</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>txt</extension>
        <mime-type>text/plain</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ulw</extension>
        <mime-type>audio/basic</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>ustar</extension>
        <mime-type>application/x-ustar</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>vxml</extension>
        <mime-type>application/voicexml+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xbm</extension>
        <mime-type>image/x-xbitmap</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xht</extension>
        <mime-type>application/xhtml+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xhtml</extension>
        <mime-type>application/xhtml+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xls</extension>
        <mime-type>application/vnd.ms-excel</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xml</extension>
        <mime-type>application/xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xpm</extension>
        <mime-type>image/x-xpixmap</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xsl</extension>
        <mime-type>application/xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xslt</extension>
        <mime-type>application/xslt+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xul</extension>
        <mime-type>application/vnd.mozilla.xul+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>xwd</extension>
        <mime-type>image/x-xwindowdump</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>vsd</extension>
        <mime-type>application/x-visio</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>wav</extension>
        <mime-type>audio/x-wav</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- Wireless Bitmap -->
        <extension>wbmp</extension>
        <mime-type>image/vnd.wap.wbmp</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- WML Source -->
        <extension>wml</extension>
        <mime-type>text/vnd.wap.wml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- Compiled WML -->
        <extension>wmlc</extension>
        <mime-type>application/vnd.wap.wmlc</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- WML Script Source -->
        <extension>wmls</extension>
        <mime-type>text/vnd.wap.wmlscript</mime-type>
    </mime-mapping>
    <mime-mapping>
        <!-- Compiled WML Script -->
        <extension>wmlscriptc</extension>
        <mime-type>application/vnd.wap.wmlscriptc</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>wmv</extension>
        <mime-type>video/x-ms-wmv</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>wrl</extension>
        <mime-type>x-world/x-vrml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>wspolicy</extension>
        <mime-type>application/wspolicy+xml</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>Z</extension>
        <mime-type>application/x-compress</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>z</extension>
        <mime-type>application/x-compress</mime-type>
    </mime-mapping>
    <mime-mapping>
        <extension>zip</extension>
        <mime-type>application/zip</mime-type>
    </mime-mapping>

  <!-- ==================== Default Welcome File List ===================== -->
  <!-- When a request URI refers to a directory, the default servlet looks  -->
  <!-- for a "welcome file" within that directory and, if present,          -->
  <!-- to the corresponding resource URI for display.  If no welcome file   -->
  <!-- is present, the default servlet either serves a directory listing,   -->
  <!-- or returns a 404 status, depending on how it is configured.          -->
  <!--                                                                      -->
  <!-- If you define welcome files in your own application's web.xml        -->
  <!-- deployment descriptor, that list *replaces* the list configured      -->
  <!-- here, so be sure that you include any of the default values that     -->
  <!-- you wish to include.                                                 -->

    <welcome-file-list>
        <welcome-file>index.html</welcome-file>
        <welcome-file>index.htm</welcome-file>
        <welcome-file>index.jsp</welcome-file>
    </welcome-file-list>

</web-app>
""".strip()

web_xml_content_missing_timeout = """
<?xml version="1.0" encoding="ISO-8859-1"?>
<web-app xmlns="http://java.sun.com/xml/ns/javaee"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://java.sun.com/xml/ns/javaee http://java.sun.com/xml/ns/javaee/web-app_2_5.xsd"
    version="2.5">

    <servlet>
        <servlet-name>default</servlet-name>
        <servlet-class>org.apache.catalina.servlets.DefaultServlet</servlet-class>
        <init-param>
            <param-name>debug</param-name>
            <param-value>0</param-value>
        </init-param>
        <init-param>
            <param-name>listings</param-name>
            <param-value>false</param-value>
        </init-param>
        <load-on-startup>1</load-on-startup>
    </servlet>
</web-app>
"""


def test_get_tmo():
    result = TomcatWebXml(context_wrap(web_xml_content))
    assert result.get("session-timeout") == 30


def test_get_tmo_missing_timeout():
    result = TomcatWebXml(context_wrap(web_xml_content_missing_timeout))
    assert result.get("session-timeout") is None


server_xml_content = """
<?xml version='1.0' encoding='utf-8'?>
<Server port="8005" shutdown="SHUTDOWN">

  <Listener className="org.apache.catalina.core.AprLifecycleListener" SSLEngine="on" />
  <Listener className="org.apache.catalina.core.JasperListener" />
  <Listener className="org.apache.catalina.core.JreMemoryLeakPreventionListener" />
  <Listener className="org.apache.catalina.mbeans.GlobalResourcesLifecycleListener" />
<Listener className="org.apache.catalina.core.ThreadLocalLeakPreventionListener" />
  <GlobalNamingResources>
    <Resource name="UserDatabase" auth="Container"
              type="org.apache.catalina.UserDatabase"
              description="User database that can be updated and saved"
              factory="org.apache.catalina.users.MemoryUserDatabaseFactory"
              pathname="conf/tomcat-users.xml" />
  </GlobalNamingResources>

  <!-- A "Service" is a collection of one or more "Connectors" that share
       a single "Container" Note:  A "Service" is not itself a "Container",
       so you may not define subcomponents such as "Valves" at this level.
       Documentation at /docs/config/service.html
   -->
  <Service name="Catalina">
    <Connector port="8080" protocol="HTTP/1.1"
               connectionTimeout="20000"
               redirectPort="8443" />

    <Connector port="8443" protocol="HTTP/1.1" SSLEnabled="true"
               maxThreads="150" scheme="https" secure="true"
               clientAuth="want"
               sslProtocols="TLSv1.2,TLSv1.1,TLSv1"
               keystoreFile="conf/keystore"
               truststoreFile="conf/keystore"
               keystorePass="oXQ8LfAGsf97KQxwwPta2X3vnUv7P5QM"
               keystoreType="PKCS12"
               ciphers="SSL_RSA_WITH_3DES_EDE_CBC_SHA,
                    TLS_RSA_WITH_AES_256_CBC_SHA,
                    TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA,
                    TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA,
                    TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA,
                    TLS_ECDH_RSA_WITH_AES_128_CBC_SHA,
                    TLS_ECDH_RSA_WITH_AES_256_CBC_SHA,
                    TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA"
               truststorePass="oXQ8LfAGsf97KQxwwPta2X3vnUv7P5QM" />

    <!-- Define an AJP 1.3 Connector on port 8009 -->
    <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" />


    <Engine name="Catalina" defaultHost="localhost">

      <Realm className="org.apache.catalina.realm.UserDatabaseRealm"
             resourceName="UserDatabase"/>

      <Host name="localhost"  appBase="webapps"
            unpackWARs="true" autoDeploy="true"
            xmlValidation="false" xmlNamespaceAware="false">

      </Host>
    </Engine>
  </Service>
</Server>
"""


def test_tomcat_server_xml():
    result = TomcatServerXml(context_wrap(server_xml_content))
    engines = result.get_elements(".//Service/Engine")
    assert len(engines) == 1
    assert engines[0].get('name') == "Catalina"


def test_web_xml_doc_examples():
    env = {
            'TomcatWebXml': TomcatWebXml,
            'web_xml': TomcatWebXml(context_wrap(web_xml_content, path='/usr/share/tomcat/web.xml')),
            'TomcatServerXml': TomcatServerXml,
            'server_xml': TomcatServerXml(context_wrap(server_xml_content, path='/usr/share/tomcat/server.xml'))
          }
    failed, total = doctest.testmod(tomcat_xml, globs=env)
    assert failed == 0



import yaml
import json
import pytest

try:
    from unittest.mock import patch, mock_open
    builtin_open = "builtins.open"
except Exception:
    from mock import patch, mock_open
    builtin_open = "__builtin__.open"

from insights import package_info
from insights.client.config import InsightsConfig
from insights.client.constants import InsightsConstants as constants
from insights.core.exceptions import SkipComponent, ContentException
from insights.specs.datasources.client_metadata import (
    ansible_host, basic_auth_insights_client, blacklist_report,
    blacklisted_specs, branch_info, display_name, egg_release,
    version_info, tags)


TAGS_YAML = """
---
group: _group-name-value_
location: _location-name-value_
description:
 - RHEL:
    - 8
    - 9
 - SAP
key 4: value
""".strip()


def test_ansible_host():
    ic = InsightsConfig(ansible_host='test_abc')
    result = ansible_host({'client_config': ic})
    assert result.content == ['test_abc']

    with pytest.raises(SkipComponent):
        ansible_host({})

    ic = InsightsConfig()
    with pytest.raises(SkipComponent):
        ansible_host({'client_config': ic})


def test_blacklist_report():
    ic = InsightsConfig(obfuscate=True, obfuscate_hostname=False)
    rm = {'patterns': {'regex': ['test', 'pwd', '12.*4', '^abcd']}}
    result = blacklist_report({'client_config': ic, 'redact_config': rm})
    assert json.loads(result.content[0]) == {
            "obfuscate": True, "obfuscate_hostname": False, "commands": 0,
            "files": 0, "components": 0, "patterns": 4, "keywords": 0,
            "using_new_format": True, "using_patterns_regex": True}

    ic = InsightsConfig(obfuscate=True, obfuscate_hostname=True)
    rm = {'patterns': ['test', 'pwd', '', '']}
    result = blacklist_report({'client_config': ic, 'redact_config': rm})
    assert json.loads(result.content[0]) == {
            "obfuscate": True, "obfuscate_hostname": True, "commands": 0,
            "files": 0, "components": 0, "patterns": 2, "keywords": 0,
            "using_new_format": True, "using_patterns_regex": False}

    result = blacklist_report({})
    assert json.loads(result.content[0]) == {
            "obfuscate": False, "obfuscate_hostname": False, "commands": 0,
            "files": 0, "components": 0, "patterns": 0, "keywords": 0,
            "using_new_format": True, "using_patterns_regex": False}


@patch("insights.specs.datasources.client_metadata.BLACKLISTED_SPECS", [])
def test_blacklisted_specs_empty():
    with pytest.raises(SkipComponent):
        blacklisted_specs({})


@patch("insights.specs.datasources.client_metadata.BLACKLISTED_SPECS", ["date", "auditd_conf"])
def test_blacklisted_specs():
    result = blacklisted_specs({})
    assert result.content == ['{"specs": ["date", "auditd_conf"]}']


def test_basic_auth_insights_client():
    ic = InsightsConfig(offline=False, username='test_username', password='test_password')
    result = basic_auth_insights_client({'client_config': ic})
    assert result.content == ['{"username_set": true, "pass_set": true}']
    assert result.path == '/basic_conf'


def test_branch_info():
    ic = InsightsConfig(offline=True)
    result = branch_info({'client_config': ic})
    assert result.content == [json.dumps(constants.default_branch_info)]

    b_info = {'test': 'v1', 'brach': 'v2'}
    ic = InsightsConfig(offline=False, branch_info=b_info)
    result = branch_info({'client_config': ic})
    assert result.content == [json.dumps(b_info)]


def test_display_name():
    ic = InsightsConfig(display_name='test_abc')
    result = display_name({'client_config': ic})
    assert result.content == ['test_abc']

    with pytest.raises(SkipComponent):
        display_name({})

    ic = InsightsConfig()
    with pytest.raises(SkipComponent):
        display_name({'client_config': ic})


@patch(builtin_open, mock_open(read_data='/testing'))
def test_egg_release():
    result = egg_release({})
    assert result.content == ['/testing']


@patch(builtin_open, mock_open(read_data=''))
def test_egg_release_empty():
    with pytest.raises(SkipComponent):
        egg_release({})


@patch("yaml.safe_load", return_value=yaml.load(TAGS_YAML, Loader=yaml.Loader))
@patch("os.path.isfile", return_value=True)
@patch(builtin_open)
def test_tags_ok(m_open, m_isfile, m_load):
    result = tags({})
    result_json = json.loads(''.join(result.content).strip())
    expected_item_1 = {'key': 'group', 'value': '_group-name-value_', 'namespace': 'insights-client'}
    expected_item_2 = {'key': 'description:RHEL', 'value': 8, 'namespace': 'insights-client'}
    expected_item_3 = {'key': 'key 4', 'value': 'value', 'namespace': 'insights-client'}
    assert expected_item_1 in result_json
    assert expected_item_2 in result_json
    assert expected_item_3 in result_json


@patch("os.path.isfile", return_value=True)
@patch(builtin_open, mock_open(read_data='---\ntest\n---'))
def test_invalid_yaml(m_isfile):
    with pytest.raises(ContentException) as ce:
        tags({})
    assert "Cannot parse" in str(ce)


@patch("yaml.safe_load", return_value=None)
@patch("os.path.isfile", return_value=True)
@patch(builtin_open)
def test_empty_yaml(m_open, m_isfile, m_load):
    with pytest.raises(ContentException) as ce:
        tags({})
    assert "Empty YAML" in str(ce)


@patch("os.path.isfile", return_value=False)
def test_tags_no_file(isfile):
    with pytest.raises(SkipComponent) as se:
        tags({})
    assert "No such file" in str(se)


def test_version_info():
    v_info = {}
    v_info['core_version'] = '{0}-{1}'.format(package_info['VERSION'],
                                              package_info['RELEASE'])
    v_info['client_version'] = None

    result = version_info({})
    assert result.content == [json.dumps(v_info)]



"""
Sample Specs, Parsers and Rules for Tests
-----------------------------------------

These specs, parsers and rules are used by some of the tests.
"""
from insights import make_fail, parser, Parser, rule
from insights.core.spec_factory import SpecSet, RegistryPoint, simple_file, glob_file


class TSpecs(SpecSet):
    sample_multioutput_file = RegistryPoint(multi_output=True)
    sample_nonexistent = RegistryPoint()
    sample_raw_file = RegistryPoint(raw=True)


class TDefaultSpecs(TSpecs):
    sample_multioutput_file = glob_file('/var/log/sample_*.log')
    sample_nonexistent = simple_file('/nonexistent_file.txt')
    sample_raw_file = simple_file('/var/log/sample_2.log')


class TParser(Parser):
    def parse_content(self, content):
        self.data = content


@parser(TSpecs.sample_multioutput_file)
class TMultioutputParser(TParser):
    pass


@parser(TSpecs.sample_nonexistent)
class TNonexistentParser(TParser):
    pass


@parser(TSpecs.sample_raw_file)
class TRawParser(TParser):
    pass


@rule(TMultioutputParser)
def report_multioutput(mop):
    return make_fail('MO_SPEC', data=mop[0].data, number=len(mop))


@rule(TNonexistentParser)
def report_nonexistent(nep):
    return make_fail('NE_SPEC', data=nep.data)


@rule(TRawParser)
def report_raw(rp):
    return make_fail('RA_SPEC', data=rp.data)



"""
Tmpfiles.d configuration - files  in ``/etc/tmpfiles.d``
========================================================

The ``TmpfilesD`` class parser provides a 'rules' list, as well as a
``find_file`` method to find rules for a particular tmp file.
"""

from .. import Parser, parser, get_active_lines
from ..specs import Specs


@parser(Specs.tmpfilesd)
class TmpFilesD(Parser):
    """
    Parse files in /etc/tmpfiles.d, /usr/lib/tmpfiles.d/, and /run/tmpfiles.d.

    This parser reads the files and records the filename in each line.

    Attributes:
        files(list): A list of the files that tmpfiles.d is managing.
        rules(list): A list of dictionaries with the values of each rule.

    Sample input::

        # /usr/lib/tmpfiles.d/dnf.conf
        r! /var/cache/dnf/*/*/download_lock.pid
        e  /var/cache/dnf/ - - - 30d

    Examples:

        >>> tmpfiles = shared[TmpFilesd][0] # List is per filename
        >>> len(tmpfiles.rules)
        2
        >>> tmpfiles.files
        ['/var/cache/dnf/*/*/download_lock.pid', '/var/cache/dnf/']
        >>> tmpfiles.rules[1]
        {'path': '/var/cache/dnf/', 'type': 'e', 'mode': '-', 'age': '30d',
         'gid': '-', 'uid': '-', 'argument': None}
        >>> tmpfiles.find_file('download_lock.pid')
        [{'path': '/var/cache/dnf/*/*/download_lock.pid', 'type': 'r!',
          'mode': None, 'age': None, 'gid': None, 'uid': None, 'argument': None}]
    """

    def parse_content(self, content):
        self.files = []
        self.rules = []
        for line in get_active_lines(content):
            linelist = line.split()
            keys = ['type', 'path', 'mode', 'uid', 'gid', 'age', 'argument']
            d = dict(zip(keys, linelist))

            # each key should be available  even if the tmpfile does not contain
            # a value for that key
            [d.update({key: d.get(key)}) for key in keys]

            self.rules.append(d)

        for i in self.rules:
            self.files.append(i['path'])

    def find_file(self, filename):
        """
        Find any rules containing the file being searched.

        This method returns a list of dictionaries where the the managed file is
        found.
        """

        matched = []

        for rule in self.rules:
            if filename.split('/')[-1] in rule['path'].split('/'):
                matched.append(rule)

        return matched



"""
Custom datasource to get the current kernel version.
"""

from insights.core.context import HostContext
from insights.core.plugins import datasource
from insights.core.exceptions import SkipComponent
from insights.parsers.uname import Uname
from insights.specs import Specs


@datasource(Uname, HostContext)
def current_version(broker):
    """
    This datasource provides the current booting kernel version.

    Sample data returned::

        '4.18.0-240.el8.x86_64'

    Returns:
        String: The current kernel version.

    Raises:
        UnameError: When there is a problem occurs with uname data.
    """

    return broker[Uname].kernel


@datasource(Specs.grubby_default_kernel, HostContext)
def default_version(broker):
    """
    This datasource provides the default kernel version.

    Sample data returned::

        '4.18.0-240.el8.x86_64'

    Returns:
        String: The default kernel version.

    Raises:
        SkipComponent: When output is empty or an error occurs
    """
    try:
        content = broker[Specs.grubby_default_kernel].content
        if len(content) == 1 and len(content[0].split()) == 1:
            default_kernel = content[0]
            return default_kernel.lstrip("/boot/vmlinuz-")
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))
    raise SkipComponent



"""
VMBus device info
=================
"""
from insights.core.plugins import combiner
from insights.parsers.sys_vmbus import SysVmbusDeviceID, SysVmbusClassID


@combiner(SysVmbusDeviceID, SysVmbusClassID)
class SysVmBusDeviceInfo(object):
    '''
    Combiner to access all the VMBus devices.

    Attributes:
        devices (list): The list is dict.

    Sample output::

        [
         {
          'device_id': '47505500-0001-0000-3130-444531444234',
          'class_id': '44c4f61d-4444-4400-9d52-802e27ede19f',
          'description': 'PCI Express pass-through'
         }
        ]

    Examples:
        >>> len(output.devices)
        2
        >>> output.devices[0].get('device_id', '')
        '47505500-0001-0000-3130-444531444234'
        >>> output.devices[0].get('class_id', '')
        '44c4f61d-4444-4400-9d52-802e27ede19f'
        >>> output.devices[0].get('description', '')
        'PCI Express pass-through'
    '''
    def __init__(self, device_id, class_id):
        self.devices = []
        for d in device_id:
            for c in class_id:
                if d.id in c.file_path:
                    self.devices.append(
                        {
                            'device_id': d.id,
                            'class_id': c.id,
                            'description': c.desc
                        }
                    )



from insights.parsers import crictl_logs
from insights.parsers.crictl_logs import CrictlLogs
from insights.tests import context_wrap
import doctest

CRICTL_LOGS = """
2021-12-21T11:12:45.854971114+01:00 Successfully copied files in /usr/src/multus-cni/rhel7/bin/ to /host/opt/cni/bin/
2021-12-21T11:12:45.995998017+01:00 2021-12-21T10:12:45+00:00 WARN: {unknown parameter "-"}
2021-12-21T11:12:46.008998978+01:00 2021-12-21T10:12:46+00:00 Entrypoint skipped copying Multus binary.
2021-12-21T11:12:46.081427544+01:00 2021-12-21T10:12:46+00:00 Attempting to find master plugin configuration, attempt 0
""".strip()


def test_crictl_logs():
    logs = CrictlLogs(context_wrap(CRICTL_LOGS))
    test_1 = logs.get('skipped copying Multus binary')
    assert 1 == len(test_1)
    test_2 = logs.get('Attempting to find master plugin configuration')
    assert 1 == len(test_2)
    assert test_2[0]['raw_message'] == '2021-12-21T11:12:46.081427544+01:00 2021-12-21T10:12:46+00:00 Attempting to find master plugin configuration, attempt 0'


def test_crictl_logs_documentation():
    failed_count, tests = doctest.testmod(
        crictl_logs,
        globs={'logs': CrictlLogs(context_wrap(CRICTL_LOGS))}
    )
    assert failed_count == 0



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import satellite_mongodb
from insights.tests import context_wrap


MONGO_PULP_STORAGE_ENGINE_OUTPUT1 = '''
MongoDB shell version v3.4.9
connecting to: mongodb://127.0.0.1:27017/pulp_database
MongoDB server version: 3.4.9
{
        "name" : "wiredTiger",
        "supportsCommittedReads" : true,
        "readOnly" : false,
        "persistent" : true
}
'''


MONGO_PULP_STORAGE_ENGINE_OUTPUT2 = '''
MongoDB shell version v3.4.9
connecting to: mongodb://127.0.0.1:27017/pulp_database
MongoDB server version: 3.4.9
'''

MONGO_PULP_STORAGE_ENGINE_OUTPUT3 = '''
MongoDB shell version v3.4.9
connecting to: mongodb://127.0.0.1:27017/pulp_database
2020-02-13T23:19:57.750-0500 W NETWORK  [thread1] Failed to connect to 127.0.0.1:27017, in(checking socket for error after poll), reason: Connection refused
2020-02-13T23:19:57.751-0500 E QUERY    [thread1] Error: couldn't connect to server 127.0.0.1:27017, connection attempt failed :
connect@src/mongo/shell/mongo.js:237:13
@(connect):1:6
exception: connect failed
'''.strip()

MONGO_PULP_STORAGE_ENGINE_OUTPUT4 = '''
MongoDB shell version v3.4.9
connecting to: mongodb://127.0.0.1:27017/pulp_database
{
    "name" wrong data
}
'''.strip()

MONGO_PULP_NON_YUM_TYPE_REPOS_OUTPUT1 = """
0
"""

MONGO_PULP_NON_YUM_TYPE_REPOS_OUTPUT2 = """
MongoDB shell version v3.4.9
connecting to: mongodb://127.0.0.1:27017/pulp_database
MongoDB server version: 3.4.9
0
"""

MONGO_PULP_NON_YUM_TYPE_REPOS_OUTPUT3 = """
MongoDB shell version v3.4.9
connecting to: mongodb://127.0.0.1:27017/pulp_database
MongoDB server version: 3.4.9
ab
"""


def test_doc_examples():
    engine_output = satellite_mongodb.MongoDBStorageEngine(context_wrap(MONGO_PULP_STORAGE_ENGINE_OUTPUT1))
    repos_output = satellite_mongodb.MongoDBNonYumTypeRepos(context_wrap(MONGO_PULP_NON_YUM_TYPE_REPOS_OUTPUT2))
    globs = {
        'satellite_storage_engine': engine_output,
        'satellite_non_yum_type_repos': repos_output
    }
    failed, tested = doctest.testmod(satellite_mongodb, globs=globs)
    assert failed == 0


def test_satellite():
    output = satellite_mongodb.MongoDBStorageEngine(context_wrap(MONGO_PULP_STORAGE_ENGINE_OUTPUT1))
    assert output['supportsCommittedReads'] == 'true'
    assert output['readOnly'] == 'false'
    assert output['persistent'] == 'true'


def test_no_storage_engine():
    with pytest.raises(SkipComponent):
        satellite_mongodb.MongoDBStorageEngine(context_wrap(MONGO_PULP_STORAGE_ENGINE_OUTPUT2))
    with pytest.raises(SkipComponent):
        satellite_mongodb.MongoDBStorageEngine(context_wrap(MONGO_PULP_STORAGE_ENGINE_OUTPUT3))
    with pytest.raises(ParseException):
        satellite_mongodb.MongoDBStorageEngine(context_wrap(MONGO_PULP_STORAGE_ENGINE_OUTPUT4))


def test_bad_yum_repos_output():
    with pytest.raises(SkipComponent):
        satellite_mongodb.MongoDBNonYumTypeRepos(context_wrap(MONGO_PULP_NON_YUM_TYPE_REPOS_OUTPUT1))
    with pytest.raises(SkipComponent):
        satellite_mongodb.MongoDBNonYumTypeRepos(context_wrap(MONGO_PULP_NON_YUM_TYPE_REPOS_OUTPUT3))



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import etcd_conf
from insights.parsers.etcd_conf import EtcdConf
from insights.tests import context_wrap

ETCD_CONF = """

[member]
ETCD_NAME=f05-h19-000-1029p.rdu2.scalelab.redhat.com
ETCD_LISTEN_PEER_URLS=https://10.1.40.235:2380
ETCD_DATA_DIR=/var/lib/etcd/
#ETCD_WAL_DIR=
ETCD_SNAPSHOT_COUNT=10000
ETCD_HEARTBEAT_INTERVAL=500

[auth]
ETCD_AUTH_TOKEN=simple

""".strip()


def test_etcd_conf_empty():
    with pytest.raises(SkipComponent):
        assert etcd_conf.EtcdConf(context_wrap('')) is None


def test_etcd_conf():
    conf = EtcdConf(context_wrap(ETCD_CONF))
    assert conf.has_option('member', 'ETCD_HEARTBEAT_INTERVAL') is True


def test_etcd_conf_documentation():
    failed_count, tests = doctest.testmod(
        etcd_conf,
        globs={'conf': EtcdConf(context_wrap(ETCD_CONF))}
    )
    assert failed_count == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import iris
from insights.tests import context_wrap


IRIS_RUNNING_1 = """

Configuration 'IRIS'   (default)
    directory:    /intersystems
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems
    conf file:    iris.cpf  (SuperServer port = 1972, WebServer = 52773)
    status:       running, since Tue Jun 27 01:55:25 2023
    state:        ok
    product:      InterSystems IRIS

Configuration 'IRIS2'
    directory:    /intersystems2
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems2
    conf file:    iris.cpf  (SuperServer port = 51773, WebServer = 52774)
    status:       running, since Thu Aug 10 02:47:27 2023
    state:        ok
    product:      InterSystems IRIS
""".strip()

IRIS_RUNNING_2 = """

Configuration 'IRIS'   (default)
    directory:    /intersystems
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems
    conf file:    iris.cpf  (SuperServer port = 1972, WebServer = 52773)
    status:       running, since Tue Jun 27 01:55:25 2023
    state:        ok
    product:      InterSystems IRIS

Configuration 'IRIS2'
    directory:    /intersystems2
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems2
    conf file:    iris.cpf  (SuperServer port = 51773, WebServer = 52774)
    status:       down, last used Thu Aug 10 07:21:10 2023
    state:        ok
    product:      InterSystems IRIS
""".strip()

IRIS_DOWN = """

Configuration 'IRIS'   (default)
    directory:    /intersystems
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems
    conf file:    iris.cpf  (SuperServer port = 1972, WebServer = 52773)
    status:       down, last used Tue Jun 27 01:50:36 2023
    product:      InterSystems IRIS

Configuration 'IRIS2'
    directory:    /intersystems2
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems2
    conf file:    iris.cpf  (SuperServer port = 51773, WebServer = 52774)
    status:       down, last used Thu Aug 10 07:21:10 2023
    state:        ok
    product:      InterSystems IRIS
""".strip()

IRIS_NO_CONTENT = """"""


IRIS_CPF = """
[ConfigFile]
Product=IRIS
Version=2023.1

[Databases]
IRISSYS=/intersystems/mgr/
IRISLIB=/intersystems/mgr/irislib/
IRISTEMP=/intersystems/mgr/iristemp/
IRISLOCALDATA=/intersystems/mgr/irislocaldata/
IRISAUDIT=/intersystems/mgr/irisaudit/
ENSLIB=/intersystems/mgr/enslib/
USER=/intersystems/mgr/user/

[Namespaces]
%SYS=IRISSYS
USER=USER

[SqlSysDatatypes]
BIGINT=%Library.BigInt
BIGINT(%1)=%Library.BigInt
BINARY=%Library.Binary(MAXLEN=1)
BINARY VARYING=%Library.Binary(MAXLEN=1)
CLOB=%Stream.GlobalCharacter
DATE=%Library.Date
DATETIME=%Library.DateTime
DATETIME2=%Library.DateTime
DEC=%Library.Numeric(MAXVAL=999999999999999,MINVAL=-999999999999999,SCALE=0)
DEC(%1)=%Library.Numeric(MAXVAL=<|'$$maxval^%apiSQL(%1,0)'|>,MINVAL=<|'$$minval^%apiSQL(%1,0)'|>,SCALE=0)
DEC(%1,%2)=%Library.Numeric(MAXVAL=<|'$$maxval^%apiSQL(%1,%2)'|>,MINVAL=<|'$$minval^%apiSQL(%1,%2)'|>,SCALE=%2)
DECIMAL=%Library.Numeric(MAXVAL=999999999999999,MINVAL=-999999999999999,SCALE=0)
DECIMAL(%1)=%Library.Numeric(MAXVAL=<|'$$maxval^%apiSQL(%1,0)'|>,MINVAL=<|'$$minval^%apiSQL(%1,0)'|>,SCALE=0)
DECIMAL(%1,%2)=%Library.Numeric(MAXVAL=<|'$$maxval^%apiSQL(%1,%2)'|>,MINVAL=<|'$$minval^%apiSQL(%1,%2)'|>,SCALE=%2)
DOUBLE=%Library.Double
DOUBLE PRECISION=%Library.Double
""".strip()

IRIS_MESSAGES_LOG = """
06/26/23-08:02:17:828 (144145) 0 [Generic.Event] Allocated 495MB shared memory
06/26/23-08:02:17:828 (144145) 0 [Generic.Event] 32MB global buffers, 80MB routine buffers, 64MB journal buffers, 4MB buffer descriptors, 300MB heap, 5MB ECP, 9MB miscellaneous
06/26/23-08:02:17:831 (144145) 0 [Crypto.IntelSandyBridgeAESNI] Intel Sandy Bridge AES-NI instructions detected.
06/26/23-08:02:17:831 (144145) 0 [SIMD] SIMD optimization level: DEFAULT X86
06/26/23-08:02:17:903 (144145) 0 [WriteDaemon.UsingWIJFile] Using WIJ file: /intersystems/mgr/IRIS.WIJ
06/26/23-08:02:17:903 (144145) 0 [Generic.Event] No journaling info from prior system
06/26/23-08:02:17:903 (144145) 0 [WriteDaemon.CreatingNewWIJ] Creating a new WIJ file
06/26/23-08:02:18:104 (144145) 0 [WriteDaemon.CreatedNewWIJ] New WIJ file created
06/26/23-08:02:18:110 (144145) 0 [Generic.Event]
Startup of InterSystems IRIS [IRIS for UNIX (Red Hat Enterprise Linux 8 for x86-64) 2023.1 (Build 235.1) Fri Jun 2 2023 13:26:47 EDT]
        in ../bin/
        with mgr: /intersystems/mgr
        with wij: /intersystems/mgr/IRIS.WIJ
        from: /intersystems/mgr/
  OS=[Linux], version=[#1 SMP Fri Sep 30 11:45:06 EDT 2022], release=[4.18.0-425.3.1.el8.x86_64], machine=[x86_64]
  nodename=[rhel8].
  numasyncwijbuf: 2, wdwrite_asyncio_max: 64, wijdirectio: on, synctype: 3
  System Initialized.
06/26/23-08:02:18:136 (144182) 0 [WriteDaemon.Started] Write daemon started.
06/26/23-08:02:18:281 (144200) 0 [Database.MountedRW] Mounted database /intersystems/mgr/ (SFN 0) read-write.
06/26/23-08:02:18:317 (144207) 0 [Utility.Event] Starting ^INSTALL procedure
""".strip()


def test_iris_list():
    iris_running = iris.IrisList(context_wrap(IRIS_RUNNING_1))
    assert len(iris_running) == 2
    assert iris_running[0]['status'] == 'running, since Tue Jun 27 01:55:25 2023'
    assert iris_running.default['status'] == 'running, since Tue Jun 27 01:55:25 2023'
    assert 'state' in iris_running[0]
    assert iris_running.is_running

    iris_running2 = iris.IrisList(context_wrap(IRIS_RUNNING_2))
    assert len(iris_running2) == 2
    assert iris_running2[0]['status'] == 'running, since Tue Jun 27 01:55:25 2023'
    assert iris_running2[1]['status'] == 'down, last used Thu Aug 10 07:21:10 2023'
    assert 'state' in iris_running2[0]
    assert iris_running2.is_running

    iris_down = iris.IrisList(context_wrap(IRIS_DOWN))
    assert len(iris_down) == 2
    assert iris_down[0]['status'] == 'down, last used Tue Jun 27 01:50:36 2023'
    assert 'state' not in iris_down[0]
    assert not iris_down.is_running


def test_fail():
    with pytest.raises(SkipComponent) as e:
        iris.IrisList(context_wrap(IRIS_NO_CONTENT))
    assert "The result is empty" in str(e)


def test_iris_cpf():
    iris_cpf = iris.IrisCpf(context_wrap(IRIS_CPF))
    assert "Databases" in iris_cpf
    assert iris_cpf.get("Databases", "IRISSYS") == "/intersystems/mgr/"
    assert iris_cpf.get("SqlSysDatatypes", "DEC") == "%Library.Numeric(MAXVAL=999999999999999,MINVAL=-999999999999999,SCALE=0)"


def test_iris_messages_log():
    log = iris.IrisMessages(context_wrap(IRIS_MESSAGES_LOG))
    assert len(log.get('Generic.Event')) == 4
    assert len(log.get('shared memory')) == 1


def test_doc_examples():
    env = {
        'iris_info': iris.IrisList(context_wrap(IRIS_RUNNING_1)),
        'iris_cpf': iris.IrisCpf(context_wrap(IRIS_CPF)),
        'iris_log': iris.IrisMessages(context_wrap(IRIS_MESSAGES_LOG)),
    }
    failed, total = doctest.testmod(iris, globs=env)
    assert failed == 0



import base64
import copy
import hashlib
import logging
import os
import pkgutil
import tempfile

import six

import insights.client.apps.ansible
from insights.client.apps.ansible.playbook_verifier.contrib import gnupg
from insights.client.apps.ansible.playbook_verifier.contrib.ruamel_yaml.ruamel import yaml
from insights.client.apps.ansible.playbook_verifier.contrib.ruamel_yaml.ruamel.yaml.comments import CommentedMap, CommentedSeq
from insights.client.apps.ansible.playbook_verifier.contrib.ruamel_yaml.ruamel.yaml.scalarint import ScalarInt
from insights.client.constants import InsightsConstants as constants


__all__ = ("load_playbook_yaml", "verify", "PlaybookVerificationError")

yaml = yaml.YAML(typ='rt')
yaml.indent(mapping=2, sequence=4, offset=2)
yaml.default_flow_style = False
yaml.preserve_quotes = True
yaml.width = 200

PLAYBOOK_SIGNATURE_LABEL = 'insights_signature'
PLAYBOOK_DYNAMIC_LABELS = ['hosts', 'vars']

PUBLIC_KEY_PATH = pkgutil.get_data(insights.client.apps.ansible.__name__, 'playbook_verifier/public.gpg')

logger = logging.getLogger(__name__)


class PlaybookVerificationError(Exception):
    """Exception raised when playbook verification fails."""
    def __init__(self, message="Could not verify Ansible playbook"):
        self.message = message
        super(PlaybookVerificationError, self).__init__(self.message)

    def __str__(self):
        return self.message


def load_playbook_yaml(playbook):
    """Parse a string into a dictionary.

    :param playbook: The Ansible playbook
    :type playbook: str

    :returns: The playbook dictionary
    :rtype: list[dict]

    :raises PlaybookVerificationError: The input is empty or corrupted.
    """
    if not playbook:
        raise PlaybookVerificationError("Cannot load a playbook from an empty input.")

    try:
        return yaml.load(playbook)
    except Exception:
        raise PlaybookVerificationError("Could not load the playbook.")


def normalize_snippet(snippet):
    """In Python 2, get rid of any default unicode values.

    :param snippet: The (part of) Ansible playbook.
    :type snippet: dict | yaml.comments.CommentedMap

    :returns: Snippet with any Unicode text converted to ASCII.
    :rtype: CommentedMap
    """
    result = CommentedMap()
    for key, value in snippet.iteritems():
        if isinstance(value, CommentedMap):
            result[key] = CommentedMap(normalize_snippet(value))
            continue

        if isinstance(value, CommentedSeq):
            new_sequence = CommentedSeq()
            for item in value:
                if isinstance(item, six.text_type):
                    new_sequence.append(item.encode('ascii', 'ignore'))
                    continue
                if isinstance(item, CommentedMap):
                    new_sequence.append(normalize_snippet(item))
                    continue
                new_sequence.append(item)

            result[key] = new_sequence
            continue

        if isinstance(value, six.text_type):
            result[key] = value.encode('ascii', 'ignore')
            continue

        if isinstance(value, ScalarInt):
            result[key] = int(value)
            continue

        result[key] = value

    return result


def serialize_playbook_snippet(snippet):
    """Convert the snippet object to bytes.

    :param snippet: The (part of) Ansible playbook.
    :type snippet: dict
    :returns: Serialized snippet.
    :rtype: bytes
    """
    if six.PY2:
        return str(normalize_snippet(snippet)).encode("utf-8")
    return str(snippet).encode("utf-8")


def hash_playbook_snippet(serialized_snippet):
    """Create SHA256 hash of a playbook snippet.

    :param serialized_snippet: The (part of) Ansible playbook.
    :type serialized_snippet: bytes
    :returns: Hashed snippet.
    :rtype: bytes
    """
    sha = hashlib.sha256()
    sha.update(serialized_snippet)
    return sha.digest()


def get_public_key(gpg):
    """Import public GPG key.

    :param gpg: Path to the GPG key.
    :type gpg: gnupg.GPG
    :returns: Import result
    :rtype: ...
    """
    if not PUBLIC_KEY_PATH:
        raise PlaybookVerificationError(message="Public key file not found.")

    public_key = PUBLIC_KEY_PATH
    import_results = gpg.import_keys(public_key)
    if import_results.count < 1:
        raise PlaybookVerificationError(message="Public key import failed.")

    return import_results


def exclude_dynamic_elements(snippet):
    """Remove dynamic elements from the Ansible playbook snippet.

    Some parts of the Ansible playbook are dynamic (e.g. host systems) and cannot be signed,
    because the signature would have to be unique for every playbook that is generated.

    This function removes these variable sections.

    :returns: New playbook dictionary without the unwanted values.
    :rtype: dict

    :raises PlaybookVerificationError:
    """
    if 'insights_signature_exclude' not in snippet.get('vars', {}):
        raise PlaybookVerificationError(
            "The playbook snippet does not have the key 'vars/insights_signature_exclude', "
            "dynamic exclusion cannot be performed."
        )

    exclusions = snippet['vars']['insights_signature_exclude'].split(',')  # type: list[str]
    result = copy.deepcopy(snippet)  # type: dict[str, ...]

    for element in exclusions:
        elements = [string for string in element.split("/") if string != '']  # type: list[str]

        if len(elements) == 1 and elements[0] in PLAYBOOK_DYNAMIC_LABELS:
            try:
                del result[elements[0]]
            except Exception:
                raise PlaybookVerificationError(
                    "Dynamic key '{key}' does not exist and could not be excluded.".format(key=element)
                )
            continue

        if len(elements) == 2 and elements[0] in PLAYBOOK_DYNAMIC_LABELS:
            try:
                del result[elements[0]][elements[1]]
            except Exception:
                raise PlaybookVerificationError(
                    "Dynamic key '{key}' does not exist and could not be excluded.".format(key=element)
                )
            continue

        raise PlaybookVerificationError(
            "Dynamic key '{key}' cannot be excluded from verification.".format(key=element)
        )

    return result


def execute_verification(snippet, encoded_signature):
    """Use GPG to verify the snippet.

    :param snippet: The (part of) Ansible playbook.
    :type snippet: dict
    :param encoded_signature: The base64-encoded signature.
    :type encoded_signature: str

    :returns: Result of the GPG verification and a hash of the snippet.
    :rtype: Tuple[..., bytes]
    """
    gpg = gnupg.GPG(gnupghome=constants.insights_core_lib_dir)
    serialized_snippet = serialize_playbook_snippet(snippet)
    snippet_hash = hash_playbook_snippet(serialized_snippet)
    decoded_signature = base64.b64decode(encoded_signature)

    # load public key
    get_public_key(gpg)

    fd, fn = tempfile.mkstemp()
    os.write(fd, decoded_signature)
    os.close(fd)

    result = gpg.verify_data(fn, snippet_hash)
    os.unlink(fn)

    return result, snippet_hash


def verify_playbook_snippet(snippet):
    """Verify the signature in a playbook.

    :param snippet: The (part of) Ansible playbook.
    :type snippet: dict

    :returns: Result of the GPG verification and a hash of the snippet.
    :rtype: Tuple[..., bytes]
    """
    if not isinstance(snippet.get("vars", None), dict):
        raise PlaybookVerificationError("The playbook snippet doesn't have a section 'vars'.")
    if snippet.get("vars", {}).get(PLAYBOOK_SIGNATURE_LABEL, None) is None:
        raise PlaybookVerificationError("The playbook snippet doesn't contain the Insights signature.")

    cleaned_snippet = exclude_dynamic_elements(snippet)  # type: dict
    encoded_signature = snippet["vars"][PLAYBOOK_SIGNATURE_LABEL]  # type: str

    return execute_verification(cleaned_snippet, encoded_signature)


def get_playbook_snippet_revocation_list(revoked_playbooks_yaml):
    """
    Load the list of revoked playbook snippet hashes from the egg.

    :param revoked_playbooks_yaml: The YAML containing revoked playbooks.
    :type revoked_playbooks_yaml: bytes
    :returns: Revocation entries in a form of `name:hash`.
    :rtype: dict[str, str]
    """
    try:
        # We know the structure of the YAML will stay like this:
        # > - name: revocation list
        # >   timestamp: ...
        # >   vars:
        # >     insights_signature_exclude: ...
        # >     insights_signature: !!binary | ...
        # >   revoked_playbooks:
        # >     - name: ...
        # >       hash: ...
        # >     - name: ...
        # >       hash: ...
        # There will never be more than one top-level object, we can safely do [0].
        revoked_playbooks = yaml.load(revoked_playbooks_yaml)[0]  # type: dict
    except Exception:
        raise PlaybookVerificationError("Could not load snippet revocation list.")

    verified, snippet_hash = verify_playbook_snippet(revoked_playbooks)

    if not verified:
        raise PlaybookVerificationError("List of revocation signatures is invalid.")

    revocation_list = revoked_playbooks.get("revoked_playbooks", [])  # type: list[str]
    return revocation_list


def verify(playbook):
    """Verify the GPG-signed Ansible playbook.

    :param playbook: Unverified playbook.
    :type playbook: dict
    :returns: Verified Ansible playbook.
    :rtype: dict
    :raises PlaybookVerificationError: An error occurred when trying to verify the playbook.
    """
    logger.info("Verifying Ansible playbook")

    if not playbook:
        raise PlaybookVerificationError("Empty playbooks cannot be verified.")

    revocation_list_file_content = pkgutil.get_data('insights', 'revoked_playbooks.yaml')  # type: bytes
    revocation_list = get_playbook_snippet_revocation_list(revocation_list_file_content)  # type: dict[str, str]

    name = playbook.get("name", "NAME UNAVAILABLE")
    verified, playbook_hash = verify_playbook_snippet(playbook)  # type: ..., str

    if not verified:
        raise PlaybookVerificationError(message="Template '{0}' has invalid signature".format(name))

    for revoked_item in revocation_list:
        if playbook_hash == bytearray.fromhex(revoked_item['hash']):
            raise PlaybookVerificationError(message="Template '{0}' is on the revoked list.".format(name))

    logger.info("All templates in the Ansible playbook were verified.")
    return playbook



"""
PostgreSQLConf - file ``/var/lib/pgsql/data/postgresql.conf``
=============================================================

The PostgreSQL configuration file is in a fairly standard 'key = value'
format, with the equals sign being optional.  A hash mark (#) marks the
rest of the line as a comment.

The configuration then appears as a dictionary in the `data` property.

This parser does not attempt to know the default value of any property; it
only shows what's defined in the configuration file as given.

This parser also provides several utility functions to make sense of values
specific to PostgreSQL.  These are:

  * `as_duration(property)`
      Convert the value (given in milliseconds, seconds, minutes, hours or
      days) to seconds (as a floating point value).
  * `as_boolean(property)`
      If the value is 'on', 'true', 'yes', or '1', return True.  If the value
      is 'off', 'false', 'no' or '0', return False.  Unique prefixes of these
      are acceptable and case is ignored.
  * `as_memory_bytes(property)`
      Convert a number given in KB, MB or GB into bytes, where 1 kilobyte is
      1024 bytes.

All three type conversion functions will raise a ValueError if the value
doesn't match the spec or cannot be converted to the correct type.

Example:
    >>> pgsql = shared[PostgreSQLConf]
    >>> 'port' in pgsql
    True
    >>> pgsql['port']
    '5432'
    >>>
"""
from .. import Parser, parser, get_active_lines, LegacyItemAccess
import re
from insights.specs import Specs


@parser(Specs.postgresql_conf)
class PostgreSQLConf(LegacyItemAccess, Parser):
    """
    Parses postgresql.conf and converts it into a dictionary of properties.
    """
    _value_error_str = "Do not recognise '{val}' for setting '{item}' " +\
                       "as a {_type}"

    def parse_content(self, content):
        """
        Parsing rules from :

        https://www.postgresql.org/docs/9.3/static/config-setting.html

        One parameter is specified per line. The equal sign between name
        and value is optional. Whitespace is insignificant and blank lines
        are ignored.   Hash marks (#) designate the remainder of the line as
        a comment. Parameter values that are not simple identifiers or
        numbers must be single-quoted. To embed a single quote in a
        parameter value, write either two quotes (preferred) or
        backslash-quote.
        """
        pg_dict = {}
        for line in get_active_lines(content):
            # Comments and blank lines removed by get_active_lines
            # Split on equals or on first word
            if '=' in line:
                key, value = [s.strip() for s in line.split("=", 1)]
            else:
                key, value = [s.strip() for s in line.split(' ', 1)]
            # If value is quoted, quotes appear first and last - remove them.
            if value[0] == "'" and value[-1] == "'":
                value = value[1:-1]
            # If value contains '' or \', change to single quote
            if "''" in value:
                value = value.replace("''", "'")
            if "\\'" in value:
                value = value.replace("\\'", "'")
            # Now save value in key
            pg_dict[key] = value
        self.data = pg_dict

    def as_duration(self, item, default=None):
        """
        Postgres's time durations for checkpoint_timeout can have 'ms', 's',
        'min', 'h', or 'd' suffixes.  We convert all of them here to seconds.

        See https://www.postgresql.org/docs/9.3/static/config-setting.html :-

        "Valid time units are ms (milliseconds), s (seconds), min (minutes),
        h (hours), and d (days)"

        We return a floating point number because of the possibility of
        convertion from milliseconds, and because maybe someone will say
        8.4h.
        """
        if not item:
            return None
        if item in self.data:
            value = self.data[item]
        else:
            value = default
            if isinstance(value, int) or isinstance(value, float):
                return float(value)
        dur_re = re.compile(r'^(?P<number>\d+)(?P<suffix>ms|s|min|h|d)?$')
        length_of = {'ms': 0.001, 's': 1, 'min': 60, 'h': 3600, 'd': 86400}

        match = dur_re.search(value)
        if match:
            # Do we have a suffix at all?  If not, assume seconds, return float
            number, suffix = match.group('number', 'suffix')
            if suffix is None:
                return float(number)
            # Do we have a matching suffix?
            # assert: suffix in length_of, due to regex
            return float(number) * length_of[suffix]
        else:
            raise ValueError(self._value_error_str.format(
                             val=value, item=item, _type='duration'
                             ))

    def as_boolean(self, item, default=None):
        """
        See https://www.postgresql.org/docs/9.3/static/config-setting.html :-

        "Boolean values can be written as on, off, true, false, yes, no, 1,
        0 (all case-insensitive) or any unambiguous prefix of these."
        """
        if not item:
            return None
        if item in self.data:
            value = self.data[item]
        else:
            value = default
            if value is None or isinstance(value, bool):
                return value

        lval = value.lower()
        if lval in ('on', 't', 'tr', 'tru', 'true', 'y', 'ye', 'yes', '1'):
            return True
        if lval in ('of', 'off', 'f', 'fa', 'fal', 'fals', 'false', 'n', 'no', '0'):
            return False
        raise ValueError(self._value_error_str.format(
                         val=value, item=item, _type='boolean'
                         ))

    def as_memory_bytes(self, item, default=None):
        """
        See https://www.postgresql.org/docs/9.3/static/config-setting.html :-

        "Valid memory units are kB (kilobytes), MB (megabytes), and GB
        (gigabytes).  Note that the multiplier for memory units is 1024, not
        1000."
        """
        if not item:
            return None
        size_of = {'kB': 1024, 'MB': 1048576, 'GB': 1048576 * 1024}

        if item in self.data:
            value = self.data[item]
        else:
            value = default

        # Don't bother to do conversions if we're already integer-esque
        if isinstance(value, int):
            return value
        elif value.isdigit():
            return int(value)

        suffix = value[-2:]
        if suffix in size_of:
            return int(value[:-2]) * size_of[suffix]
        else:
            raise ValueError(self._value_error_str.format(
                             val=value, item=item, _type='memory unit'
                             ))



#!/usr/bin/env python
"""
Example Cluster Rule - allnodes_cpu
===================================

This example demonstrates a custome spec, parser, facts, combiner
and multiple cluster rules.  It can be run using the command::

    $ insights-run -p examples.cluster_rules.allnodes_cpu \
            -i topology_allnodes \
            examples/cluster_rules/cluster_hosts.tar.gz

or from the examples/cluster_rules directory::

    $ ./allnodes_cpu.py -i topology_allnodes \
            examples/cluster_rules/cluster_hosts.tar.gz
"""
from __future__ import print_function
from colorama import Fore, Style  # noqa: F401
from insights import make_response
from insights.core import YAMLParser
from insights.core.cluster import ClusterMeta
from insights.core.plugins import combiner, fact, parser, rule
from insights.core.spec_factory import SpecSet, simple_file
from insights.parsers.cpuinfo import CpuInfo

PODS_PER_CORE = 10
MAX_PODS = 250
MASTER_MIN_CORE = 4


# Define a datasource for node-config.yaml since core doesn't ship with one
class Specs(SpecSet):
    """ Specs to collect data the cluster hosts """
    node_config_yaml = simple_file("etc/origin/node/node-config.yaml")


# Parse the yaml
@parser(Specs.node_config_yaml)
class NodeConfig(YAMLParser):
    """
    Parser to parse the contents of the node_config_yaml spec

    Implementation is provided by the YAMLParser class.
    """
    pass


# A fact is generated from every node in the cluster
# Return from it a dictionary or list of dictionaries. You'll access these
# later
@fact(CpuInfo, NodeConfig)
def cluster_info(cpu, cfg):
    """
    Collects fact for each host

    Collects the cpu and node configuration facts to be used by the rule.

    Arguments:
        cpu (CpuInfo): Parser object for the cpu info.
        cfg (NodeConfig): Parser object for the node configuration.

    Returns:
        dict: Dictionary of fact information including the keys
        ``cpu_count``, ``pods_per_core_int``, ``pods_per_core_customized``,
        ``max_pods``, and ``max_pods_customized``.
    """
    cpus = cpu.cpu_count
    pods_per_core = cfg.doc.find("pods-per-core")
    pods_per_core_int = int(pods_per_core.value) if pods_per_core else PODS_PER_CORE
    cfg_max_pods = cfg.doc.find("max-pods")
    cfg_max_pods_int = int(cfg_max_pods.value) if cfg_max_pods else MAX_PODS
    calc_max_pods = cpus * pods_per_core_int

    return {
        "cpu_count": cpus,
        "pods_per_core": pods_per_core_int,
        "pods_per_core_customized": bool(pods_per_core),
        "max_pods": min(cfg_max_pods_int, calc_max_pods),
        "max_pods_customized": bool(cfg_max_pods)
    }


def master_etcd(info, meta, max_pod_cluster, label):
    """ Function used to create the response for all master node types """
    nodes = meta.get(label, []) or []
    info = info[info["machine_id"].isin(nodes)]
    if info.empty:
        return

    cpu_factor = max_pod_cluster / 1000.0
    nocpu_expected = MASTER_MIN_CORE + (max_pod_cluster / 1000.0)
    bad = info[info["cpu_count"] < nocpu_expected]
    good = info[info["cpu_count"] >= nocpu_expected]
    return make_response("MASTER_ETCD",
                         nocpu_expected=nocpu_expected, cpu_factor=cpu_factor,
                         bad=bad, good=good, max_pod_cluster=max_pod_cluster,
                         GREEN=Fore.GREEN, RED=Fore.RED, YELLOW=Fore.YELLOW, NC=Style.RESET_ALL)


def infra_nodes(info, meta, max_pod_cluster, label, key):
    """ Function used to create the response for all infra node types """
    nodes = meta.get(label, []) or []
    infos = info[info["machine_id"].isin(nodes)]
    if infos.empty:
        return
    return make_response(key, max_pod_cluster=max_pod_cluster, infos=infos,
                         GREEN=Fore.GREEN, RED=Fore.RED, YELLOW=Fore.YELLOW, NC=Style.RESET_ALL)


@combiner(cluster_info, cluster=True)
def calc_max_pos_cluster(info):
    """
    Combiner summarizes cluster info after cluster_info facts
    have been collected for all cluster hosts.global
    """
    return info["max_pods"].sum()


@rule(cluster_info, calc_max_pos_cluster, ClusterMeta, cluster=True)
def report_master(info, max_pod_cluster, meta):
    """ Rule to report for master node types """
    return master_etcd(info, meta, max_pod_cluster, "master")


@rule(cluster_info, calc_max_pos_cluster, ClusterMeta, cluster=True)
def report_etcd(info, max_pod_cluster, meta):
    """ Rule to report for etcd node types """
    return master_etcd(info, meta, max_pod_cluster, "etcd")


@rule(cluster_info, calc_max_pos_cluster, ClusterMeta, cluster=True)
def report_infra(info, max_pod_cluster, meta):
    """ Rule to report for infra node types """
    return infra_nodes(info, meta, max_pod_cluster, "infra", "INFRA")


@rule(cluster_info, calc_max_pos_cluster, ClusterMeta, cluster=True)
def report_nodes(info, max_pod_cluster, meta):
    """ Rule to report for nodes node types """
    return infra_nodes(info, meta, max_pod_cluster, "nodes", "NODES")


#
# Jinga templates for all output
#
MASTER_CONTENT = """
{% for idx, row in good.iterrows() -%}
{{row.machine_id}}: {{GREEN}}[passed]{{NC}}
{% endfor %}
{% for idx, row in bad.iterrows() -%}
{{row.machine_id}}: {{RED}}[failed]{{NC}} - reason: {{row.cpu_count}} is less than the expected {{nocpu_expected}} cores
{% endfor %}
""".strip()

NODE_CONTENT = """
{% for idx, row in infos.iterrows() %}
{{row.machine_id}}:
CPUS: {{row.cpu_count}}
{%- if row.pods_per_core_customized -%} {{YELLOW}} pods-per-core customized: {{row.pods_per_core}}{{NC}} {% endif %}
{%- if row.max_pods_customized -%} {{YELLOW}} max-pods customized: {{row.max_pods}}{{NC}} {% endif %}
max pods: {{row.max_pods}}
{% endfor %}
""".strip()

CONTENT = {
    "MASTER_ETCD": MASTER_CONTENT,
    "INFRA": NODE_CONTENT,
    "NODES": NODE_CONTENT + """
================================
max app pods cluster: {{max_pod_cluster}}
""".strip()
}

if __name__ == "__main__":
    from insights import run
    run([report_master, report_etcd, report_infra, report_nodes], print_summary=True)



from insights.parsr import Char


def boom(_):
    raise Exception("Boom")


def test_error():
    p = Char("a").map(boom)
    try:
        p("a")
    except Exception as ex:
        assert "boom" in str(ex)



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import grubby
from insights.parsers.grubby import GrubbyDefaultIndex, GrubbyDefaultKernel
from insights.tests import context_wrap

DEFAULT_INDEX_1 = '0'
DEFAULT_INDEX_2 = '1'
ABDEFAULT_INDEX_EMPTY = ''
DEFAULT_INDEX_AB = '-2'

DEFAULT_KERNEL = "/boot/vmlinuz-2.6.32-573.el6.x86_64"
DEFAULT_KERNEL_EMPTY = ""
DEFAULT_KERNEL_AB = """
/boot/vmlinuz-2.6.32-573.el6.x86_64"
/boot/vmlinuz-2.6.32-573.el6.x86_64"
""".strip()
DEFAULT_KERNEL_INVALID = 'rpm-sort: Invalid input'
DEFAULT_KERNEL_INVALID_2 = 'rpm-sort: Invalid input '
DEFAULT_KERNEL_INVALID_3 = """
/usr/libexec/grubby/grubby-bls: fork: retry: Resource temporarily unavailable
/usr/libexec/grubby/grubby-bls: fork: retry: Resource temporarily unavailable
""".strip()

DEFAULT_KERNEL_WITH_ERRORS_MSGS_1 = """
/usr/libexec/grubby/grubby-bls: fork: retry: Resource temporarily unavailable
/usr/libexec/grubby/grubby-bls: fork: retry: Resource temporarily unavailable
/usr/libexec/grubby/grubby-bls: fork: retry: Resource temporarily unavailable
/usr/libexec/grubby/grubby-bls: fork: retry: Resource temporarily unavailable
/boot/vmlinuz-4.18.0-147.5.1.el8_1.x86_64
""".strip()
DEFAULT_KERNEL_WITH_ERRORS_MSGS_2 = """
grub2-editenv: error: cannot rename the file /boot/grub2/grubenv.new to /boot/grub2/grubenv: No such file or directory.
/boot/vmlinuz-3.10.0-862.el7.x86_64
""".strip()
DEFAULT_KERNEL_WITH_ERRORS_MSGS_3 = """
grep: /boot/grub2/grubenv: No such file or directory
/boot/cm/images/dell-rhel8.2/boot/vmlinuz-4.18.0-348.23.1.el8_5.x86_64
""".strip()
DEFAULT_KERNEL_WITH_ERRORS_MSGS_4 = """
/etc/os-release: line 5: VERSION_ID-Peter=8.7: command not found
/etc/os-release: line 6: PLATFORM_ID-Peter=platform:el8: command not found
/boot/vmlinuz-4.18.0-425.10.1.el8_7.x86_64
""".strip()


def test_grubby_default_index():
    res = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1))
    assert res.default_index == 0

    res = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_2))
    assert res.default_index == 1


def test_grubby_default_index_ab():
    with pytest.raises(SkipComponent) as excinfo:
        GrubbyDefaultIndex(context_wrap(ABDEFAULT_INDEX_EMPTY))
    assert 'Empty output' in str(excinfo.value)

    with pytest.raises(ParseException) as excinfo:
        GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_AB))
    assert 'Invalid output:' in str(excinfo.value)


def test_grubby_default_kernel():
    res = GrubbyDefaultKernel(context_wrap(DEFAULT_KERNEL))
    assert res.default_kernel == DEFAULT_KERNEL

    content_with_error_msgs = [
        DEFAULT_KERNEL_WITH_ERRORS_MSGS_1,
        DEFAULT_KERNEL_WITH_ERRORS_MSGS_2,
        DEFAULT_KERNEL_WITH_ERRORS_MSGS_3,
        DEFAULT_KERNEL_WITH_ERRORS_MSGS_4,
    ]
    for content in content_with_error_msgs:
        this_res = GrubbyDefaultKernel(context_wrap(content))
        assert this_res.default_kernel == content.split()[-1].strip()


def test_grubby_default_kernel_ab():
    with pytest.raises(SkipComponent) as excinfo:
        GrubbyDefaultKernel(context_wrap(DEFAULT_KERNEL_EMPTY))
    assert 'Empty output' in str(excinfo.value)

    with pytest.raises(ParseException) as excinfo:
        GrubbyDefaultKernel(context_wrap(DEFAULT_KERNEL_AB))
    assert 'Invalid output:' in str(excinfo.value)

    with pytest.raises(ParseException) as excinfo:
        GrubbyDefaultKernel(context_wrap(DEFAULT_KERNEL_INVALID))
    assert 'Invalid output:' in str(excinfo.value)

    with pytest.raises(ParseException) as excinfo:
        GrubbyDefaultKernel(context_wrap(DEFAULT_KERNEL_INVALID_2))
    assert 'Invalid output:' in str(excinfo.value)

    with pytest.raises(ParseException) as excinfo:
        GrubbyDefaultKernel(context_wrap(DEFAULT_KERNEL_INVALID_3))
    assert 'Invalid output:' in str(excinfo.value)


def test_doc_examples():
    env = {
            'grubby_default_index': GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1)),
            'grubby_default_kernel': GrubbyDefaultKernel(context_wrap(DEFAULT_KERNEL)),
          }
    failed, total = doctest.testmod(grubby, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import libssh_config
from insights.parsers.libssh_config import LibsshConfig
from insights.tests import context_wrap

CLIENT_CONFIG = """
# Parse system-wide crypto configuration file
Include /etc/crypto-policies/back-ends/libssh.config
# Parse OpenSSH configuration file for consistency
Include /etc/ssh/ssh_config
""".strip()

SERVER_CONFIG = """
# Parse system-wide crypto configuration file
Include /etc/crypto-policies/back-ends/libssh.config
# Parse OpenSSH configuration file for consistency
Include /etc/ssh/sshd_config
""".strip()


def test_config_no_data():
    with pytest.raises(SkipComponent):
        LibsshConfig(context_wrap(""))


def test_constructor():
    result = LibsshConfig(context_wrap(CLIENT_CONFIG))

    assert 'Include' in result
    assert len(result['Include']) == 2
    assert result['Include'][0] == '/etc/crypto-policies/back-ends/libssh.config'
    assert result['Include'][1] == '/etc/ssh/ssh_config'


def test_doc_examples():
    env = {
        "config": LibsshConfig(context_wrap(SERVER_CONFIG)),
    }
    failed, total = doctest.testmod(libssh_config, globs=env)
    assert failed == 0



import doctest

from insights.parsers import ls_sys_firmware
from insights.parsers.ls_sys_firmware import LsSysFirmware
from insights.tests import context_wrap


LS_SYS_FIRMWARE = """
/sys/firmware:
total 0
drwxr-xr-x.  5 0 0 0 Dec 22 17:56 .
dr-xr-xr-x. 13 0 0 0 Dec 22 17:56 ..
drwxr-xr-x.  5 0 0 0 Dec 22 17:56 acpi
drwxr-xr-x.  3 0 0 0 Dec 22 17:57 dmi
drwxr-xr-x. 10 0 0 0 Dec 22 17:57 memmap

/sys/firmware/acpi:
total 0
drwxr-xr-x. 5 0 0    0 Dec 22 17:56 .
drwxr-xr-x. 5 0 0    0 Dec 22 17:56 ..
drwxr-xr-x. 6 0 0    0 Feb 10 15:54 hotplug
drwxr-xr-x. 2 0 0    0 Feb 10 15:54 interrupts
-r--r--r--. 1 0 0 4096 Feb 10 15:54 pm_profile
drwxr-xr-x. 3 0 0    0 Dec 22 17:56 tables
"""


def test_ls_sys_firmware():
    ls = LsSysFirmware(context_wrap(LS_SYS_FIRMWARE))
    assert "acpi" not in ls
    assert "/sys/firmware/acpi" in ls
    assert ls.dirs_of("/sys/firmware") == ['.', '..', 'acpi', 'dmi', 'memmap']
    assert ls.files_of("/sys/firmware/acpi") == ['pm_profile']


def test_doc_examples():
    env = {'ls_sys_firmware': LsSysFirmware(context_wrap(LS_SYS_FIRMWARE))}
    failed, total = doctest.testmod(ls_sys_firmware, globs=env)
    assert failed == 0



"""
NamedCheckconf parser - command ``named-checkconf -p``
======================================================

Named-checkconf is a syntax checking tool for named configuration file.
Named is a name server used by BIND.
"""

import re

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs

# regex for matching 'dnssec-enable no'
DNSSEC_DISABLED = re.compile(r'dnssec-enable\s+no;')
# regex for matching 'disable-algorithms' section
DISABLE_ALGORITHMS = re.compile(r'disable-algorithms[^}]*};')
# regex for matching 'disable-ds-digests' section
DISABLE_DS_DIGESTS = re.compile(r'disable-ds-digests[^}]*};')
# regex for matching values in single or double quotation marks
INNER_VALLUES = re.compile(r'(?:\"|\')(.*)(?:\"|\')')


@parser(Specs.named_checkconf_p)
class NamedCheckconf(CommandParser):
    """
    Class for parsing the ``named-checkconf -p`` command.

    Attributes:
        is_dnssec_disabled (bool): True, if dnssec is not enabled, False otherwise.
        dnssec_line (string): The line which disabled dnssec, if it is not enabled, None otherwise.
        disable_algorithms (dict): Dictionary where the key is a domain and
                                   the value is a list of all algorithms associated with it.
        disable_ds_digests (dict): Dictionary where the key is a domain and
                                   the value is a list of all digests associated with it.

    Raises:
        SkipComponent: When content is empty or cannot be parsed.

    Sample output of this command is::

        logging {
            channel "default_debug" {
                file "data/named.run";
                severity dynamic;
            };
        };
        options {
            directory "/var/named";
            dump-file "/var/named/data/cache_dump.db";
            listen-on port 53 {
                127.0.0.1/32;
            };
            listen-on-v6 port 53 {
                ::1/128;
            };
            managed-keys-directory "/var/named/dynamic";
            memstatistics-file "/var/named/data/named_mem_stats.txt";
            pid-file "/run/named/named.pid";
            recursing-file "/var/named/data/named.recursing";
            secroots-file "/var/named/data/named.secroots";
            session-keyfile "/run/named/session.key";
            statistics-file "/var/named/data/named_stats.txt";
            disable-algorithms "." {
                "RSAMD5";
                "DSA";
            };
            disable-ds-digests "." {
                "GOST";
            };
            dnssec-enable yes;
            dnssec-validation yes;
            recursion yes;
            allow-query {
                "localhost";
            };
        };
        managed-keys {
            "." initial-key 257 3 8 "AwEAAagAIKlVZrpC6Ia7gEzahOR+9W29euxhJhVVLOyQbSEW0O8gcCjF
                        FVQUTf6v58fLjwBd0YI0EzrAcQqBGCzh/RStIoO8g0NfnfL2MTJRkxoX
                        bfDaUeVPQuYEhg37NZWAJQ9VnMVDxP/VHL496M/QZxkjf5/Efucp2gaD
                        X6RS6CXpoY68LsvPVjR0ZSwzz1apAzvN9dlzEheX7ICJBBtuA6G3LQpz
                        W5hOA2hzCTMjJPJ8LbqF6dsV6DoBQzgul0sGIcGOYl7OyQdXfZ57relS
                        Qageu+ipAdTTJ25AsRTAoub8ONGcLmqrAmRLKBP1dfwhYB4N7knNnulq
                        QxA+Uk1ihz0=";
            "." initial-key 257 3 8 "AwEAAaz/tAm8yTn4Mfeh5eyI96WSVexTBAvkMgJzkKTOiW1vkIbzxeF3
                        +/4RgWOq7HrxRixHlFlExOLAJr5emLvN7SWXgnLh4+B5xQlNVz8Og8kv
                        ArMtNROxVQuCaSnIDdD5LKyWbRd2n9WGe2R8PzgCmr3EgVLrjyBxWezF
                        0jLHwVN8efS3rCj/EWgvIWgb9tarpVUDK/b58Da+sqqls3eNbuv7pr+e
                        oZG+SrDK6nWeL3c6H5Apxz7LjVc1uTIdsIXxuOLYA4/ilBmSVIzuDWfd
                        RUfhHdY6+cn8HFRm+2hM8AnXGXws9555KrUB5qihylGa8subX2Nn6UwN
                        R1AkUTV74bU=";
        };
        zone "." IN {
            type hint;
            file "named.ca";
        };
        zone "localhost.localdomain" IN {
            type master;
            file "named.localhost";
            allow-update {
                "none";
            };
        };
        zone "localhost" IN {
            type master;
            file "named.localhost";
            allow-update {
                "none";
            };
        };
        zone "1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa" IN {
            type master;
            file "named.loopback";
            allow-update {
                "none";
            };
        };
        zone "1.0.0.127.in-addr.arpa" IN {
            type master;
            file "named.loopback";
            allow-update {
                "none";
            };
        };
        zone "0.in-addr.arpa" IN {
            type master;
            file "named.empty";
            allow-update {
                "none";
            };
        };

    Examples:
        >>> type(named_checkconf)
        <class 'insights.parsers.named_checkconf.NamedCheckconf'>
        >>> named_checkconf.is_dnssec_disabled
        False
        >>> named_checkconf.dnssec_line is None
        True
        >>> named_checkconf.disable_algorithms
        {'.': ['RSAMD5', 'DSA']}
        >>> named_checkconf.disable_ds_digests
        {'.': ['GOST']}
    """

    def __init__(self, context):
        self.is_dnssec_disabled = False  # dnssec is enabled by default
        self.dnssec_line = None
        self.disable_algorithms = {}
        self.disable_ds_digests = {}
        super(NamedCheckconf, self).__init__(context)

    def parse_content(self, content):
        if not content:
            raise SkipComponent('No content.')

        full_result = '\n'.join(content)

        match_dnssec = DNSSEC_DISABLED.search(full_result)
        if match_dnssec:
            self.is_dnssec_disabled = True
            self.dnssec_line = match_dnssec.group(0)

        self.disable_algorithms = self.retrieve_disabled(DISABLE_ALGORITHMS, full_result)
        self.disable_ds_digests = self.retrieve_disabled(DISABLE_DS_DIGESTS, full_result)

    def retrieve_disabled(self, section_regex, source):
        """
        Parses 'disable-algorithms' or 'disable_ds_digests' section into a dictionary,
        where the key is a domain and the value is a list of all algorithms/digests associated with it.

        Attributes:
            section_regex (string): The regular expression for a given section.
            source (string): The source in which a given section is searched for.
        """
        dict_of_sections = dict()
        for match_entry in section_regex.finditer(source):
            entry = match_entry.group(0)
            # collects all values in quotation marks for given section
            entry_values = [match_value.group(1) for match_value in INNER_VALLUES.finditer(entry)]
            dict_of_sections[entry_values[0]] = entry_values[1:]

        return dict_of_sections



"""
Tmpfilesd configuration
=======================

The tmpfilesd files are normally available to rules as a list of TmpFilesd
objects. This combiner turns those into one set of data, and provides a
``find_file()`` method to search for that filename among all the files.
"""

from insights.core.plugins import combiner
from insights.parsers.tmpfilesd import TmpFilesD


@combiner(TmpFilesD)
class AllTmpFiles(object):
    """
    Combiner for accessing all the tmpfilesd configuration files. Configuration
    files can be found in three directories: ``/usr/lib/tmpfiles.d``,
    ``/run/tmpfiles.d``, and ``/etc/tmpfiles.d``. Each directory overrides the
    settings in the previous directory. This combiner checks for and accounts
    for this behavior.

    Attributes:
        files(set): the set of files found in all data files.
        active_rules(dict): a dictionary of rules using the config file as the key
        file_paths(list): a list of the file paths for the configurations.
    """
    def __init__(self, tmpfiles):
        active_rules = {}
        files = set([])
        file_paths = []
        # Files must be sorted so that /etc/tmpfiles.d overrides /run/tmpfiles.d
        # which overides /usr/lib/tmpfiles.d.
        for tmpfile in sorted(tmpfiles, key=lambda f: f.file_path):
            file_paths.append(tmpfile.file_path)
            if not active_rules.get(tmpfile.file_path):
                active_rules[tmpfile.file_path] = []
            for new_rule in tmpfile.rules:
                _file = (new_rule['path'])
                if _file not in files:
                    files.add(_file)
                    active_rules[tmpfile.file_path].append(new_rule)

        self.active_rules = active_rules
        self.files = files
        self.file_paths = file_paths
        super(AllTmpFiles, self).__init__()

    def find_file(self, path):
        """
        Find all the rules matching a given file. Uses the rules dictionary to search
        so duplicate files are alreayd removed.

        Examples:
            >>> data = shared[AllTmpFiles]
            >>> results = data.find_file('/tmp/sap.conf')
            >>> len(results)
            1
            >>> results
            {'/etc/tmpfiles.d/sap.conf': {'path': '/tmp/sap.conf', 'mode': '644', 'type': 'x', 'age': None,
             'gid': None, 'uid': None, 'argument': None}}

        Parameters:
            path(str): path to be searched for among the rules.

        Returns:
            (dict): a dictionary of rules where the path is found using the config
            file path as the key.
        """
        match = {}
        for r, d in self.active_rules.items():
            for i in self.active_rules[r]:
                if i['path'] == path:
                    match[r] = i

        return match



import doctest

from insights.combiners import sysctl_conf
from insights.parsers import sysctl
from insights.tests import context_wrap


SYSCTL_CONF_TEST1 = """
# sysctl.conf sample
#
  kernel.domainname = example.com
# kernel.domainname.invalid = notvalid.com

; this one has a space which will be written to the sysctl!
  kernel.modprobe = /sbin/mod probe
""".strip()

SYSCTL_CONF_TEST2 = """
kernel.sysrq = 1
kernel.panic_on_oops = 1
kernel.panic_on_unrecovered_nmi = 1
kernel.softlockup_panic = 1
kernel.unknown_nmi_panic = 1
kernel.nmi_watchdog = 1
""".strip()

SYSCTL_CONF_TEST3 = """
vm.dirty_ratio = 15
vm.dirty_background_ratio = 5
vm.swappiness = 10
"""

SYSCTL_CONF_TEST4 = """
vm.dirty_ratio = 30
vm.dirty_background_ratio = 50
vm.swappiness = 60
kernel.sysrq = 16
kernel.domainname = test.example.com
"""

SYSCTL1_CONF_FILENAME = "/etc/sysctl.d/10-test.conf"
SYSCTL2_CONF_FILENAME = "/etc/sysctl.d/20-test.conf"
SYSCTL3_CONF_FILENAME = "/etc/sysctl.d/30-test.conf"
SYSCTL4_CONF_FILENAME = "/etc/sysctl.d/99-test.conf"
SYSCTL5_CONF_FILENAME = "/etc/sysctl.conf"
SYSCTL6_CONF_FILENAME = "/usr/lib/sysctl.d/99-test.conf"


def test_all_sysctl_conf():
    conf_1 = sysctl.SysctlConf(context_wrap(SYSCTL_CONF_TEST1, path=SYSCTL5_CONF_FILENAME))
    conf_2 = sysctl.SysctlDConfEtc(context_wrap(SYSCTL_CONF_TEST2, path=SYSCTL2_CONF_FILENAME))
    conf_3 = sysctl.SysctlDConfEtc(context_wrap(SYSCTL_CONF_TEST3, path=SYSCTL3_CONF_FILENAME))
    r = sysctl_conf.SysctlConfs(conf_1, None, [conf_2, conf_3])

    assert r['kernel.domainname'] == 'example.com'
    assert r['kernel.modprobe'] == '/sbin/mod probe'
    assert 'kernel.domainname.invalid' not in r
    assert r.search("dirty") == {"vm.dirty_ratio": "15", "vm.dirty_background_ratio": "5"}
    assert r.search("test") == {}


def test_all_sysctl_conf_different_order():
    conf_1 = sysctl.SysctlConf(context_wrap(SYSCTL_CONF_TEST1, path=SYSCTL5_CONF_FILENAME))
    conf_2 = sysctl.SysctlDConfUsr(context_wrap(SYSCTL_CONF_TEST2, path=SYSCTL6_CONF_FILENAME))
    conf_3 = sysctl.SysctlDConfEtc(context_wrap(SYSCTL_CONF_TEST3, path=SYSCTL2_CONF_FILENAME))
    conf_4 = sysctl.SysctlDConfEtc(context_wrap(SYSCTL_CONF_TEST4, path=SYSCTL4_CONF_FILENAME))
    r = sysctl_conf.SysctlConfs(conf_1, [conf_2], [conf_3, conf_4])

    assert r['kernel.domainname'] == 'example.com'
    assert r['kernel.sysrq'] == '16'
    assert r.search("dirty") == {"vm.dirty_ratio": "30", "vm.dirty_background_ratio": "50"}


def test_docs():
    conf_1 = sysctl.SysctlDConfEtc(context_wrap(SYSCTL_CONF_TEST1, path=SYSCTL1_CONF_FILENAME))
    conf_2 = sysctl.SysctlDConfEtc(context_wrap(SYSCTL_CONF_TEST2, path=SYSCTL2_CONF_FILENAME))
    conf_3 = sysctl.SysctlDConfEtc(context_wrap(SYSCTL_CONF_TEST3, path=SYSCTL3_CONF_FILENAME))

    env = {
        'sysctl_conf': sysctl_conf.SysctlConfs(None, None, [conf_1, conf_2, conf_3]),
    }
    failed, total = doctest.testmod(sysctl_conf, globs=env)
    assert failed == 0



"""
RpmPkgs - datasource ``rpm_pkgs``
=================================

Parsers for CVE-2021-35937, CVE-2021-35938, and CVE-2021-35939.
For more information, see the ``rpm_pkgs`` datasource.
"""

from insights import Parser, parser
from insights.specs import Specs


@parser(Specs.rpm_pkgs)
class RpmPkgsWritable(Parser):
    """
    Class for enabling the data from the ``rpm_pkgs`` datasource.
    It replaces the original RpmPkgs parser.

    Sample input of this parser is::

        ["httpd-core|httpd-core-2.4.53-7.el9.x86_64|Red Hat, Inc."]

    Sample output of this parser is::

        [("httpd-core", "httpd-core-2.4.53-7.el9.x86_64", "Red Hat, Inc.")]

    Examples:

        >>> type(rpm_pkgs_writable)
        <class 'insights.parsers.rpm_pkgs.RpmPkgsWritable'>
        >>> rpm_pkgs_writable.packages
        [('httpd-core', 'httpd-core-2.4.53-7.el9.x86_64', 'Red Hat, Inc.')]
    """

    def parse_content(self, content):
        self.packages = [tuple(pkg.split('|')) for pkg in content]



from __future__ import print_function
from collections import OrderedDict
from itertools import groupby
from operator import itemgetter

from insights import make_info, make_fail, make_response, make_pass
from insights.formats import FormatterAdapter
from insights.formats.template import TemplateFormat


class HtmlFormat(TemplateFormat):
    """
    This class prints a html summary of rule hits. It should be used
    as a context manager and given an instance of an
    ``insights.core.dr.Broker``. ``dr.run`` should be called within the context
    using the same broker.

    Args:
        broker (Broker): the broker to watch and provide a summary about.
        stream (file-like): Output is written to stream. Defaults to sys.stdout.
    """

    TEMPLATE = """
<!doctype html>
<html lang="en">
  <head>
    <!-- Required meta tags -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    <!-- Bootstrap CSS -->
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/css/bootstrap.min.css" integrity="sha384-MCw98/SFnGE8fJT3GXwEOngsV7Zt27NXFoaoApmYm81iuXoPkFOJwJ8ERdknLPMO" crossorigin="anonymous">

    <title>{{root}}</title>
  </head>
  <body>
    <div class="container">
      <p>
        <h2>Analysis of {{root}}</h2>
        <h4>Performed at {{start_time}} UTC</h4>
      </p>
      <div class="card">
        <div class="card-header">System Information</div>
        <div class="card-body">
        <pre>
{% for rule in rules.make_info %}
{{-rule.body}}
{% endfor -%}
        </pre>
        </div>
      </div>
      <div class="card">
        <div class="card-header">Rule Results</div>
        <div class="card-body">
          <div class="accordion" id="ruleAccordion">
          {%- for group, results in rules.items() %}
          {%- if group != "make_info" %}
          {%- for rule in results %}
            <div class="card">
              <div class="card-header bg-{% if group == "make_pass" %}success{% else %}danger{% endif %}" id="heading_{{rule.id}}">
                <h2 class="mb-0">
                  <button class="btn btn-{% if group == "make_pass" %}success{% else %}danger{% endif %} text-white" type="button" data-toggle="collapse" data-target="#{{rule.id}}" aria-expanded="true" aria-controls="{{rule.id}}">
                  {{rule.name}}
                  </button>
                </h2>
              </div>
              <div id="{{rule.id}}" class="collapse" aria-labelledby="heading_{{rule.id}}" data-parent="#ruleAccordion">
                <div class="card-body">
                <pre>
{{rule.body}}
                </pre>
                <hr />
                Contributing data:
                <ol>
                {% for d in rule.datasources %}
                  <li>
                  {{d}}
                  </li>
                {%- endfor %}
                </ol>
                <hr />
                Links:
                <ul>
                {% for cat, links in rule.links.items() %}
                  <li>{{cat}}
                  <ul>
                  {% for link in links %}
                  <li><a href="{{link}}">{{link}}</a></li>
                  {% endfor %}
                  </ul>
                  </li>
                {%- endfor %}
                </ul>
                <hr />
                Rule source: {{rule.source_path}}
                <hr />
          Documentation:
                <pre>
{{rule.mod_doc}}
{{rule.rule_doc}}
                </pre>
                </div>
              </div>
            </div>
          {%- endfor %}
          {% endif %}
          {% endfor %}
          </div>
        </div>
      </div>
    </div>
    <!-- Optional JavaScript -->
    <!-- jQuery first, then Popper.js, then Bootstrap JS -->
    <script
        src="http://code.jquery.com/jquery-3.4.1.slim.min.js"
        integrity="sha256-pasqAKBDmFT4eHoN2ndd6lN370kFiGUFyTiUHWhU7k8="
        crossorigin="anonymous"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.3/umd/popper.min.js" integrity="sha384-ZMP7rVo3mIykV+2+9J3UJ46jBk0WLaUAdn689aCwoqbBJiSnjAK/l8WvCWPIPm49" crossorigin="anonymous"></script>
    <script src="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/js/bootstrap.min.js" integrity="sha384-ChfqqxuZUCnJSK3+MXmPNIyE6ZbWh2IMqE241rYiqJxyMiZ6OW/JmZQ5stwEULTy" crossorigin="anonymous"></script>
  </body>
</html>
    """.strip()

    def create_template_context(self):
        ctx = {
            "root": self.find_root() or "Unknown",
            "start_time": self.start_time.strftime("%Y-%m-%d %H:%M:%S")
        }
        sorted_rules = {}
        response_type_getter = itemgetter("response_type")

        self.rules = sorted(self.rules, key=response_type_getter)
        for response_type, rules in groupby(self.rules, response_type_getter):
            rules = sorted(rules, key=itemgetter("name"))
            sorted_rules[response_type] = rules

        ctx["rules"] = OrderedDict()
        for key in (make_info, make_fail, make_response, make_pass):
            name = key.__name__
            if name in sorted_rules:
                ctx["rules"][name] = sorted_rules[name]
        return ctx


# this connects the formatter to the insights run CLI bits
class HtmlFormatterAdapter(FormatterAdapter):
    """ Displays results in html format. """

    def preprocess(self, broker):
        self.formatter = HtmlFormat(broker)
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()



from insights.parsers import init_process_cgroup
from insights.tests import context_wrap


CGROUP_HOST = """
11:hugetlb:/
10:memory:/
9:devices:/
8:pids:/
7:perf_event:/
6:net_prio,net_cls:/
5:blkio:/
4:freezer:/
3:cpuacct,cpu:/
2:cpuset:/
1:name=systemd:/
""".strip()


CGROUP_CONTAINER = """
11:hugetlb:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
10:memory:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
9:devices:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
8:pids:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
7:perf_event:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
6:net_prio,net_cls:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
5:blkio:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
4:freezer:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
3:cpuacct,cpu:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
2:cpuset:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
1:name=systemd:/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope
""".strip()


CGROUP_CONTAINER_1 = """
12:freezer:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
11:pids:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
10:cpuset:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
9:memory:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
8:devices:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
7:blkio:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
6:cpu,cpuacct:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
5:rdma:/
4:net_cls,net_prio:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
3:perf_event:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
2:hugetlb:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
1:name=systemd:/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope
""".strip()


def test_init_process_cgroup():
    result = init_process_cgroup.InitProcessCgroup(context_wrap(CGROUP_HOST))
    assert result.data["memory"] == ["10", "/"]
    assert result.is_container is False

    result = init_process_cgroup.InitProcessCgroup(context_wrap(CGROUP_CONTAINER))
    assert result.data["memory"] == ["10", "/system.slice/docker-55b2b88feeb4fc56bb9384e55100a8581271ca7a22399c6ec52784a35dba933b.scope"]
    assert result.is_container is True

    result = init_process_cgroup.InitProcessCgroup(context_wrap(CGROUP_CONTAINER_1))
    assert result.data["memory"] == ["9", "/machine.slice/libpod-af097fa761cd92daf9ac2e18b6d59959b1212e12621a0951223f9f55d99b452c.scope"]
    assert result.is_container is True



from insights.core.exceptions import SkipComponent
from insights.parsers import postfix_conf
from insights.parsers.postfix_conf import PostfixMaster
from insights.tests import context_wrap
import doctest
import pytest

POSTFIXMASTER = """
# ==========================================================================
# service type  private unpriv  chroot  wakeup  maxproc command + args
#               (yes)   (yes)   (no)    (never) (100)
# ==========================================================================
smtp      inet  n       -       n       -       -       smtpd
#smtp      inet  n       -       n       -       1       postscreen
pickup    unix  n       -       n       60      1       pickup
cleanup   unix  n       -       n       -       0       cleanup
qmgr      unix  n       -       n       300     1       qmgr
#qmgr     unix  n       -       n       300     1       oqmgr
tlsmgr    unix  -       -       n       1000?   1       tlsmgr
rewrite   unix  -       -       n       -       -       trivial-rewrite
bounce    unix  -       -       n       -       0       bounce
defer     unix  -       -       n       -       0       bounce
trace     unix  -       -       n       -       0       bounce
verify    unix  -       -       n       -       1       verify
flush     unix  n       -       n       1000?   0       flush
proxymap  unix  -       -       n       -       -       proxymap
proxywrite unix -       -       n       -       1       proxymap
smtp      unix  -       -       n       -       -       smtp
submission inet n       -       n       -       -       smtpd
  -o syslog_name=postfix/submission
  -o smtpd_tls_security_level=encrypt
  -o smtpd_sasl_auth_enable=yes
  -o smtpd_tls_auth_only=yes
  -o smtpd_reject_unlisted_recipient=no
  -o smtpd_client_restrictions=$mua_client_restrictions
  -o smtpd_helo_restrictions=$mua_helo_restrictions
  -o smtpd_sender_restrictions=$mua_sender_restrictions
  -o smtpd_recipient_restrictions=
  -o smtpd_relay_restrictions=permit_sasl_authenticated,reject
  -o milter_macro_daemon_name=ORIGINATING
relay     unix  -       -       n       -       -       smtp
        -o syslog_name=postfix/$service_name
#       -o smtp_helo_timeout=5 -o smtp_connect_timeout=5
showq     unix  n       -       n       -       -       showq
error     unix  -       -       n       -       -       error
retry     unix  -       -       n       -       -       error
discard   unix  -       -       n       -       -       discard
local     unix  -       n       n       -       -       local
virtual   unix  -       n       n       -       -       virtual
lmtp      unix  -       -       n       -       -       lmtp
anvil     unix  -       -       n       -       1       anvil
mailman   unix  -       n       n       -       -       pipe
  flags=FRX user=list argv=/usr/lib/mailman/bin/postfix-to-mailman.py
  ${nexthop} ${user}
""".strip()

POSTFIXMASTER_ERR = """
""".strip()

POSTFIXMASTER_DOC = """
# ==========================================================================
# service type  private unpriv  chroot  wakeup  maxproc command + args
#               (yes)   (yes)   (no)    (never) (100)
# ==========================================================================
smtp      inet  n       -       n       -       -       smtpd
pickup    unix  n       -       n       60      1       pickup
cleanup   unix  n       -       n       -       0       cleanup
qmgr      unix  n       -       n       300     1       qmgr
#qmgr     unix  n       -       n       300     1       oqmgr
tlsmgr    unix  -       -       n       1000?   1       tlsmgr
rewrite   unix  -       -       n       -       -       trivial-rewrite
bounce    unix  -       -       n       -       0       bounce
defer     unix  -       -       n       -       0       bounce
trace     unix  -       -       n       -       0       bounce
verify    unix  -       -       n       -       1       verify
flush     unix  n       -       n       1000?   0       flush
proxymap  unix  -       -       n       -       -       proxymap
proxywrite unix -       -       n       -       1       proxymap
smtp      unix  -       -       n       -       -       smtp
relay     unix  -       -       n       -       -       smtp
        -o syslog_name=postfix/$service_name
#       -o smtp_helo_timeout=5 -o smtp_connect_timeout=5
showq     unix  n       -       n       -       -       showq
error     unix  -       -       n       -       -       error
retry     unix  -       -       n       -       -       error
discard   unix  -       -       n       -       -       discard
local     unix  -       n       n       -       -       local
virtual   unix  -       n       n       -       -       virtual
lmtp      unix  -       -       n       -       -       lmtp
anvil     unix  -       -       n       -       1       anvil
scache    unix  -       -       n       -       1       scache
""".strip()


def test_postfix_master():
    results = PostfixMaster(context_wrap(POSTFIXMASTER))
    assert len(results) == 25
    assert results[0]['service'] == 'smtp'
    assert results[0]['type'] == 'inet'
    assert results[0]['private'] == 'n'
    assert results[0]['command'] == 'smtpd'
    assert results[14]['args'] == ['-o syslog_name=postfix/submission',
                                        '-o smtpd_tls_security_level=encrypt',
                                        '-o smtpd_sasl_auth_enable=yes',
                                        '-o smtpd_tls_auth_only=yes',
                                        '-o smtpd_reject_unlisted_recipient=no',
                                        '-o smtpd_client_restrictions=$mua_client_restrictions',
                                        '-o smtpd_helo_restrictions=$mua_helo_restrictions',
                                        '-o smtpd_sender_restrictions=$mua_sender_restrictions',
                                        '-o smtpd_recipient_restrictions=',
                                        '-o smtpd_relay_restrictions=permit_sasl_authenticated,reject',
                                        '-o milter_macro_daemon_name=ORIGINATING']
    assert results[-1] == {'service': 'mailman', 'type': 'unix', 'private': '-', 'unpriv': 'n', 'chroot': 'n', 'wakeup': '-', 'maxproc': '-', 'command': 'pipe', 'args': ['flags=FRX user=list argv=/usr/lib/mailman/bin/postfix-to-mailman.py', '${nexthop} ${user}']}


def test_postfix_master_err():
    with pytest.raises(SkipComponent):
        postfix_conf.PostfixMaster(context_wrap(POSTFIXMASTER_ERR))


def test_postfix_master_doc_examples():
    env = {
        'postfix_master': PostfixMaster(context_wrap(POSTFIXMASTER_DOC))
    }
    failed, total = doctest.testmod(postfix_conf, globs=env)
    assert failed == 0



import pytest
import doctest
from insights.parsers import hostname
from insights.parsers.hostname import Hostname, HostnameShort, HostnameDefault
from insights.tests import context_wrap

HOSTNAME_FULL = "rhel7.example.com"
HOSTNAME_FULL_MULTILINE = """
rhel7.example.com
"""

HOSTNAME_SHORT = "rhel7"
HOSTNAME_SHORT_MULTILINE = """
rhel7
"""


def test_full_hostname():
    data = Hostname(context_wrap(HOSTNAME_FULL))
    assert data.fqdn == "rhel7.example.com"
    assert data.raw == "rhel7.example.com"
    assert data.hostname == "rhel7"
    assert data.domain == "example.com"
    assert "{0}".format(data) == "<raw: rhel7.example.com, hostname: rhel7, domain: example.com>"


def test_full_multiline_hostname():
    data = Hostname(context_wrap(HOSTNAME_FULL_MULTILINE, strip=False))
    assert data.fqdn == "rhel7.example.com"
    assert data.hostname == "rhel7"
    assert data.domain == "example.com"
    assert "{0}".format(data) == "<raw: rhel7.example.com, hostname: rhel7, domain: example.com>"


def test_short_hostname():
    data = HostnameShort(context_wrap(HOSTNAME_SHORT))
    assert data.raw == "rhel7"
    assert data.hostname == "rhel7"
    assert "{0}".format(data) == "<raw: rhel7, hostname: rhel7>"


def test_short_multiline_hostname():
    data = HostnameShort(context_wrap(HOSTNAME_SHORT_MULTILINE, strip=False))
    assert data.raw == "rhel7"
    assert data.hostname == "rhel7"
    assert "{0}".format(data) == "<raw: rhel7, hostname: rhel7>"


def test_default_hostname():
    data = HostnameDefault(context_wrap(HOSTNAME_SHORT))
    assert data.raw == "rhel7"
    assert data.hostname == "rhel7"
    assert "{0}".format(data) == "<raw: rhel7, hostname: rhel7>"


def test_default_multiline_hostname():
    data = HostnameDefault(context_wrap(HOSTNAME_SHORT_MULTILINE, strip=False))
    assert data.raw == "rhel7"
    assert data.hostname == "rhel7"
    assert "{0}".format(data) == "<raw: rhel7, hostname: rhel7>"


def test_empty_hostname():
    with pytest.raises(Exception):
        Hostname(context_wrap(""))


def test_hostname_doc():
    env = {
            'hostname': Hostname(context_wrap(HOSTNAME_FULL)),
            'hostname_def': HostnameDefault(context_wrap(HOSTNAME_SHORT)),
            'hostname_s': HostnameShort(context_wrap(HOSTNAME_SHORT)),
          }
    failed, total = doctest.testmod(hostname, globs=env)
    assert failed == 0



"""
LsVarLog - command ``ls -laR /var/log``
=======================================

This parser reads the ``/var/log`` directory listings and uses the FileListing
parser class to provide a common access to them.

Examples:

    >>> varlog = shared[LsVarLog]
    >>> '/var/log' in varlog
    True
    >>> varlog.dir_contains('/var/log', 'messages')
    True
    >>> messages = varlog.dir_entry('/var/log', 'messages')
    >>> messages['type']
    '-'
    >>> messages['perms']
    'rw-------'
"""

from .. import FileListing, parser, CommandParser

from insights.util.file_permissions import FilePermissions
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ls_var_log)
class LsVarLog(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSla` instead (NOT ``LSlaR``).

    A parser for accessing "ls -laR /var/log".
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsVarLog, "Please use the :class:`insights.parsers.ls.LSla` instead.", "3.5.0")
        super(LsVarLog, self).__init__(*args, **kwargs)

    def get_filepermissions(self, dir_name_where_to_search, dir_or_file_name_to_get):
        """
        Returns a FilePermissions object, if found, for the specified dir or
        file name in the specified directory. The directory must be specified
        by the full path without trailing slash. The dir or file name to get
        must be specified by the name only (without path).

        This is provided for several parsers which rely on this functionality,
        and may be deprecated and removed in the future.

        Args:
            dir_name_where_to_search (string): Full path without trailing
                slash where to search.
            dir_or_file_name_to_getl (string): Name of the dir or file to get
                FilePermissions for.

        Returns:
            FilePermissions: If found or None if not found.
        """
        if dir_name_where_to_search in self:
            d = self.listings[dir_name_where_to_search]['entries']
            if dir_or_file_name_to_get in d:
                return FilePermissions(d[dir_or_file_name_to_get]['raw_entry'])
